# read emldetect source list with "raw rows" and combine them to the final
# ESA (C) 2000-2018 
#
# This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
#
#    SAS is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    SAS is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
# stacked source list with summary rows per observation and instrument

sub stack_sourcelist() {

    # expecting at least three arguments:
    @_>=3 or SAS::message($SAS::AppMsg, $SAS::SilentMsg, 
                         "Internal code error: function stack_sourcelist() ".
                         "called with wrong number of arguments. ");

    # 1. mandatory: file name of input emldetect source list
    my $inlist = $_[0];

    # 2. mandatory: file name of stacked output source list
    my $outlist = $_[1];
    my $outlistsum;

    # 3. mandatory: minimum likelihood per pointing
    my $mindetml = $_[2];

    # 4. optional: hash with header keywords
    # may include hrpndef, hrm1def, hrm2def, information for
    # the catalogue columns and a comment
    my %hhead = %{$_[3]};

#    my @addpar = @_[4..$#_];

   ########################################
    # INTERNAL PARAMETERS OF SAS TASKS
    # - NEED TO BE UPDATED IN CASE THE TASKS ARE UPDATED
    my $maskthr = 0.15;                      # psfthreshold in emldetect; FIXED
    # dpssflag (i.e. SSC internally only):
    my $flag_extlimit = 1.5;
    my $flag_mfraclimit=0.5;
    my $flag_brfactor=65;
    my $flag_brlimit=1000;
    my $flag_minrad=10;
    my $flag_maxrad=400;
    my $flag_extfactor=3;
    my $flag_maxextrad=200;
    my $flag_fixext=160;
    my $flag_brexfraction=0.4;
    my $flag_detmlfraction=0.9;
    my $flag_badfactor=1;

    ########################################

    # null values
    my $mynull = 0**(-1);           # floating point
    my %mynulls;                    # all types
    $mynulls{"E"} = $mynull;        # real
    $mynulls{"D"} = $mynull;        # double
    $mynulls{"B"} = -128;           # byte
    $mynulls{"I"} = -32768;         # short
    $mynulls{"J"} = -2147483648;    # integer
    $mynulls{"K"} = -1;             # long - currently only SRCID
    $mynulls{"A"} = "";             # string (without length information)

    ########################################
    ## variables:
    my $infits;       ## handler of the input fits file
    my $emlfits;      ## handler of the output fits file
    my $fullheader;   ## structure holding the header of the primary extension
    my $listheader;   ## structure holding the header of the srclist extension
    my %srclist;      ## structure of arrays holding the srclist table
    my @srcinids;     ## source identifiers in the input source list
    my @srcnames;     ## array of the column names of the output source list
    my @srcvalinds;   ## array of valid row indexes (after source selection)
    my $nbands;       ## number of energy bands used during source detection
    my $frstrow;      ## index of first row of currently addressed detection
    my $thisrow;      ## index of currently addressed row in the source list
    my @newvals;      ## array of new values in a column and a row slice
    my %instrows;     ## hash of arrays holding the row numbers per instrument
    my %hrdef;        ## energy bands used to calculate HRs
    my @pointings;    ## pointings per source
    ## declared later-on:
    # - modified columns of the source list
    # - $srcid:          ML_SRC_ID of the currently addressed detection
    # - @srcinds:        indexes of all rows of $srcid
    # - $instslice:      pointer to instrument IDs of these rows
    # - $instidpnt:      ID_INST of currently addressed pointing
    # - @pntinds:        row indexes of this pointing; subset of @srcinds
    # - @instinds:       row indexes of the instrument; subset of @srcinds
    # - @pntrows:        row indexes of all pointings with same ID_BAND
    my @rowinds;      ## used in different contexts
    ## variables to store intermediate results
    my %seenids;      ## used when determining the unique detection IDs
    my $tmpexpmap;    ## temporary EXP_MAP  values; used for checks
    my $tmpmaskfrac;  ## temporary MASKFRAC values; used for checks
    my $tmpdetml;     ## detection likelihood per pointing
    my @tmpmaxdetml;  ## used to calculate maximum of pointing likelihoods
    my $tmpflux;      ## flux values; used to calculate the total flux
    my $tmpfluxerr;   ## flux errors; used to calculate the total flux error
    my $tmprate;      ## rate values; used to calculate the total rate
    my $tmprateerr;   ## rate errors; used to calculate the total rate error
    my @tmpnblend;    ## number of simultaneously fit sources
    my @tmphrval=($mynull,$mynull,$mynull,$mynull); ## hardness ratios
    my @tmphrerr=($mynull,$mynull,$mynull,$mynull); ## their errors
    my @tmpontime=(0,0,0);                     ## summed ontime per instrument
    my @tmppileup=($mynull,$mynull,$mynull);   ## max. pileup per instrument
    my $tmpflaginst;  ## used to merge the camera flags
    my @tmpflagband;  ## used to merge the camera flags
    my @tmpflagpnt;   ## used to merge the camera flags
    ## declared later-on:
    # - %seenpnts:       used when determining the instrument IDs per detection
    # - @instmaskfracs:  maxima of all pointing MASKFRACs
    # - @instfluxvals:   total flux per instrument and energy band
    # - @instfluxerrs:   flux error per instrument and energy band
    # - @instratevals:   total rate per instrument and energy band
    # - @instrateerrs:   rate error per instrument and energy band
    my $startcol=6;   ## number of first column which will get new values
    my $nnewrows = 4; ## number of new summary rows per pointing       ### FIXED
    my $status=0;     ## exit status of fits routines

    ########################################
    # summary source list with catalogue-like columns
    my $sumfits;      ## fits handler
    my %suminfo;      ## holding the columns of the summary source list
    my @sumnames;     ## array of the column names of the summary list
    my @sumlist;      ## array of hashes with values for summary source list
    my $scnt=-1;      ## current row number within sumlist
    my $sum_nobs     = 0;  ## number of observations in stack
    my $sum_ncontrib = 0;  ## number of contributing observations per source
    my $sum_nexp     = 0;  ## number of contributing instruments  per source
    my $ra_ref       = 0;  ## reference coordinates
    my $de_ref       = 0;  ## reference coordinates
    my $dist_ref     = 0;  ## distance to reference coordinates
    my %fhash;        ## used to pass information to subroutine ltvar
    my %fminmax;      ## used to pass information to subroutine ltvar
    my @ltvarout;     ## temporarily holding the results of ltvar
    # determine file base
    my ($suffix) =  ($outlist =~ /.*(\..*)/) 
                    ? $outlist =~ /.*(\..*)/ : "";
    if ($suffix eq ".gz" || $suffix eq ".GZ") {
        ($suffix) = ($outlist =~ /.*(\..*)\.(.*)/)
                    ? $outlist =~ /.*(\..*)\.(.*)/ : "";
    }
    ($outlistsum) = ($outlist =~ /(.*)$suffix/);
    $outlistsum   = $outlistsum."_sum".$suffix;
    ########################################

    my @insts = ( "EP", "PN", "M1", "M2" );

    # column names (most of them follow the 3XMM convention)
    @sumnames = ( "SRCID"        ,    # $sum_srcid
                  "OBS_ID"       ,
                  "ML_ID_SRC"    ,
                  "BOX_ID_SRC"   ,
                  "N_OBS"        ,    # $sum_obs
                  "N_CONTRIB"    ,    # $sum_ncontrib
                  "N_EXP"        ,    # $sum_nexp
                  "RA"           ,
                  "DEC"          ,
                  "RADEC_ERR"    ,
                  "LII"          ,
                  "BII"          ,
                  "X_IMA"        ,
                  "X_IMA_ERR"    ,
                  "Y_IMA"        ,
                  "Y_IMA_ERR"    ,
                  "DIST_NN"      ,
                  "N_BLEND"      ,    # Number of simultaneously fit sources
                  "EP_FLUX"      ,
                  "EP_FLUX_ERR"  ,
                  "EP_1_FLUX"    ,
                  "EP_1_FLUX_ERR",
                  "EP_2_FLUX"    ,
                  "EP_2_FLUX_ERR",
                  "EP_3_FLUX"    ,
                  "EP_3_FLUX_ERR",
                  "EP_4_FLUX"    ,
                  "EP_4_FLUX_ERR",
                  "EP_5_FLUX"    ,
                  "EP_5_FLUX_ERR",
                  "PN_FLUX"      ,
                  "PN_FLUX_ERR"  ,
                  "PN_1_FLUX"    ,
                  "PN_1_FLUX_ERR",
                  "PN_2_FLUX"    ,
                  "PN_2_FLUX_ERR",
                  "PN_3_FLUX"    ,
                  "PN_3_FLUX_ERR",
                  "PN_4_FLUX"    ,
                  "PN_4_FLUX_ERR",
                  "PN_5_FLUX"    ,
                  "PN_5_FLUX_ERR",
                  "M1_FLUX"      ,
                  "M1_FLUX_ERR"  ,
                  "M1_1_FLUX"    ,
                  "M1_1_FLUX_ERR",
                  "M1_2_FLUX"    ,
                  "M1_2_FLUX_ERR",
                  "M1_3_FLUX"    ,
                  "M1_3_FLUX_ERR",
                  "M1_4_FLUX"    ,
                  "M1_4_FLUX_ERR",
                  "M1_5_FLUX"    ,
                  "M1_5_FLUX_ERR",
                  "M2_FLUX"      ,
                  "M2_FLUX_ERR"  ,
                  "M2_1_FLUX"    ,
                  "M2_1_FLUX_ERR",
                  "M2_2_FLUX"    ,
                  "M2_2_FLUX_ERR",
                  "M2_3_FLUX"    ,
                  "M2_3_FLUX_ERR",
                  "M2_4_FLUX"    ,
                  "M2_4_FLUX_ERR",
                  "M2_5_FLUX"    ,
                  "M2_5_FLUX_ERR",
                  "EP_RATE"      ,
                  "EP_RATE_ERR"  ,
                  "PN_RATE"      ,
                  "PN_RATE_ERR"  ,
                  "PN_1_RATE"    ,
                  "PN_1_RATE_ERR",
                  "PN_2_RATE"    ,
                  "PN_2_RATE_ERR",
                  "PN_3_RATE"    ,
                  "PN_3_RATE_ERR",
                  "PN_4_RATE"    ,
                  "PN_4_RATE_ERR",
                  "PN_5_RATE"    ,
                  "PN_5_RATE_ERR",
                  "M1_RATE"      ,
                  "M1_RATE_ERR"  ,
                  "M1_1_RATE"    ,
                  "M1_1_RATE_ERR",
                  "M1_2_RATE"    ,
                  "M1_2_RATE_ERR",
                  "M1_3_RATE"    ,
                  "M1_3_RATE_ERR",
                  "M1_4_RATE"    ,
                  "M1_4_RATE_ERR",
                  "M1_5_RATE"    ,
                  "M1_5_RATE_ERR",
                  "M2_RATE"      ,
                  "M2_RATE_ERR"  ,
                  "M2_1_RATE"    ,
                  "M2_1_RATE_ERR",
                  "M2_2_RATE"    ,
                  "M2_2_RATE_ERR",
                  "M2_3_RATE"    ,
                  "M2_3_RATE_ERR",
                  "M2_4_RATE"    ,
                  "M2_4_RATE_ERR",
                  "M2_5_RATE"    ,
                  "M2_5_RATE_ERR",
                  "EP_CTS"       ,
                  "EP_CTS_ERR"   ,
                  "PN_CTS"       ,
                  "PN_CTS_ERR"   ,
                  "M1_CTS"       ,
                  "M1_CTS_ERR"   ,
                  "M2_CTS"       ,
                  "M2_CTS_ERR"   ,
                  "EP_DET_ML"    ,
                  "PN_DET_ML"    ,
                  "PN_1_DET_ML"  ,
                  "PN_2_DET_ML"  ,
                  "PN_3_DET_ML"  ,
                  "PN_4_DET_ML"  ,
                  "PN_5_DET_ML"  ,
                  "M1_DET_ML"  ,
                  "M1_1_DET_ML"  ,
                  "M1_2_DET_ML"  ,
                  "M1_3_DET_ML"  ,
                  "M1_4_DET_ML"  ,
                  "M1_5_DET_ML"  ,
                  "M2_DET_ML"    ,
                  "M2_1_DET_ML"  ,
                  "M2_2_DET_ML"  ,
                  "M2_3_DET_ML"  ,
                  "M2_4_DET_ML"  ,
                  "M2_5_DET_ML"  ,
                  "EXTENT"       ,
                  "EXTENT_ERR"   ,
                  "EXTENT_ML"    ,
                  "EP_HR1"       ,
                  "EP_HR1_ERR"   ,
                  "EP_HR2"       ,
                  "EP_HR2_ERR"   ,
                  "EP_HR3"       ,
                  "EP_HR3_ERR"   ,
                  "EP_HR4"       ,
                  "EP_HR4_ERR"   ,
                  "PN_HR1"       ,
                  "PN_HR1_ERR"   ,
                  "PN_HR2"       ,
                  "PN_HR2_ERR"   ,
                  "PN_HR3"       ,
                  "PN_HR3_ERR"   ,
                  "PN_HR4"       ,
                  "PN_HR4_ERR"   ,
                  "M1_HR1"       ,
                  "M1_HR1_ERR"   ,
                  "M1_HR2"       ,
                  "M1_HR2_ERR"   ,
                  "M1_HR3"       ,
                  "M1_HR3_ERR"   ,
                  "M1_HR4"       ,
                  "M1_HR4_ERR"   ,
                  "M2_HR1"       ,
                  "M2_HR1_ERR"   ,
                  "M2_HR2"       ,
                  "M2_HR2_ERR"   ,
                  "M2_HR3"       ,
                  "M2_HR3_ERR"   ,
                  "M2_HR4"       ,
                  "M2_HR4_ERR"   ,
                  "PN_EXP"       ,
                  "PN_1_EXP"     ,
                  "PN_2_EXP"     ,
                  "PN_3_EXP"     ,
                  "PN_4_EXP"     ,
                  "PN_5_EXP"     ,
                  "M1_EXP"       ,
                  "M1_1_EXP"     ,
                  "M1_2_EXP"     ,
                  "M1_3_EXP"     ,
                  "M1_4_EXP"     ,
                  "M1_5_EXP"     ,
                  "M2_EXP"       ,
                  "M2_1_EXP"     ,
                  "M2_2_EXP"     ,
                  "M2_3_EXP"     ,
                  "M2_4_EXP"     ,
                  "M2_5_EXP"     ,
                  "PN_BG"        ,
                  "PN_1_BG"      ,
                  "PN_2_BG"      ,
                  "PN_3_BG"      ,
                  "PN_4_BG"      ,
                  "PN_5_BG"      ,
                  "M1_BG"        ,
                  "M1_1_BG"      ,
                  "M1_2_BG"      ,
                  "M1_3_BG"      ,
                  "M1_4_BG"      ,
                  "M1_5_BG"      ,
                  "M2_BG"        ,
                  "M2_1_BG"      ,
                  "M2_2_BG"      ,
                  "M2_3_BG"      ,
                  "M2_4_BG"      ,
                  "M2_5_BG"      ,
                  "EP_ONTIME"    ,
                  "PN_ONTIME"    ,
                  "M1_ONTIME"    ,
                  "M2_ONTIME"    ,
                  "PN_PILEUP"    ,
                  "M1_PILEUP"    ,
                  "M2_PILEUP"    ,
                  "PN_MASKFRAC"  ,
                  "M1_MASKFRAC"  ,
                  "M2_MASKFRAC"  ,
                  "DIST_REF"     ,    # angular distance to the ref. coordinates
                  "EP_OFFAX"     ,
                  "PN_OFFAX"     ,
                  "M1_OFFAX"     ,
                  "M2_OFFAX"     ,
                  "PN_1_VIG"     ,
                  "PN_2_VIG"     ,
                  "PN_3_VIG"     ,
                  "PN_4_VIG"     ,
                  "PN_5_VIG"     ,
                  "M1_1_VIG"     ,
                  "M1_2_VIG"     ,
                  "M1_3_VIG"     ,
                  "M1_4_VIG"     ,
                  "M1_5_VIG"     ,
                  "M2_1_VIG"     ,
                  "M2_2_VIG"     ,
                  "M2_3_VIG"     ,
                  "M2_4_VIG"     ,
                  "M2_5_VIG"     ,
                  "STACK_FLAG"   ,  
                  "EP_FLAG"      ,  
                  "PN_FLAG"      ,  
                  "M1_FLAG"      ,  
                  "M2_FLAG"      ,
		  "OVERLAP"      ,    # $sum_ncontrib>1
                  "VAR_CHI2"      ,
                  "VAR_CHI2_1"    ,
                  "VAR_CHI2_2"    ,
                  "VAR_CHI2_3"    ,
                  "VAR_CHI2_4"    ,
                  "VAR_CHI2_5"    ,
                  "VAR_PROB"      ,
                  "VAR_PROB_1"    ,
                  "VAR_PROB_2"    ,
                  "VAR_PROB_3"    ,
                  "VAR_PROB_4"    ,
                  "VAR_PROB_5"    ,
                  "FRATIO"       ,
                  "FRATIO_ERR"   ,
                  "FRATIO_1"     ,
                  "FRATIO_1_ERR" ,
                  "FRATIO_2"     ,
                  "FRATIO_2_ERR" ,
                  "FRATIO_3"     ,
                  "FRATIO_3_ERR" ,
                  "FRATIO_4"     ,
                  "FRATIO_4_ERR" ,
                  "FRATIO_5"     ,
                  "FRATIO_5_ERR" ,
                  "FLUXVAR"      ,
                  "FLUXVAR_1"    ,
                  "FLUXVAR_2"    ,
                  "FLUXVAR_3"    ,
                  "FLUXVAR_4"    ,
                  "FLUXVAR_5"    ,
                  "MJD_FIRST"    ,  # headers (DATE-OBS & xmmtimeconv)
                  "MJD_LAST"     ,  # headers (DATE-END & xmmtimeconv)
                  "REVOLUT"      ,  # headers
                  "PA_PNT"       ,  # headers
                  "PN_SUBMODE"   ,  # headers
                  "M1_SUBMODE"   ,  # headers
                  "M2_SUBMODE"   ,  # headers
                  "PN_FILTER"    ,  # headers
                  "M1_FILTER"    ,  # headers
                  "M2_FILTER"       # headers
        );

    # formats
    $suminfo{form}{"SRCID"        } = "K";
    $suminfo{form}{"OBS_ID"       } = "10A";
    $suminfo{form}{"ML_ID_SRC"    } = "J";
    $suminfo{form}{"BOX_ID_SRC"   } = "J";
    $suminfo{form}{"N_OBS"        } = "J";
    $suminfo{form}{"N_CONTRIB"    } = "J";
    $suminfo{form}{"N_EXP"        } = "J";
    $suminfo{form}{"RA"           } = "D";
    $suminfo{form}{"DEC"          } = "D";
    $suminfo{form}{"RADEC_ERR"    } = "E";
    $suminfo{form}{"LII"          } = "D";
    $suminfo{form}{"BII"          } = "D";
    $suminfo{form}{"X_IMA"        } = "E";
    $suminfo{form}{"X_IMA_ERR"    } = "E";
    $suminfo{form}{"Y_IMA"        } = "E";
    $suminfo{form}{"Y_IMA_ERR"    } = "E";
    $suminfo{form}{"DIST_NN"      } = "E";
    $suminfo{form}{"N_BLEND"      } = "J";
    $suminfo{form}{"EP_FLUX"      } = "E";
    $suminfo{form}{"EP_FLUX_ERR"  } = "E";
    $suminfo{form}{"EP_1_FLUX"    } = "E";
    $suminfo{form}{"EP_1_FLUX_ERR"} = "E";
    $suminfo{form}{"EP_2_FLUX"    } = "E";
    $suminfo{form}{"EP_2_FLUX_ERR"} = "E";
    $suminfo{form}{"EP_3_FLUX"    } = "E";
    $suminfo{form}{"EP_3_FLUX_ERR"} = "E";
    $suminfo{form}{"EP_4_FLUX"    } = "E";
    $suminfo{form}{"EP_4_FLUX_ERR"} = "E";
    $suminfo{form}{"EP_5_FLUX"    } = "E";
    $suminfo{form}{"EP_5_FLUX_ERR"} = "E";
    $suminfo{form}{"PN_FLUX"      } = "E";
    $suminfo{form}{"PN_FLUX_ERR"  } = "E";
    $suminfo{form}{"PN_1_FLUX"    } = "E";
    $suminfo{form}{"PN_1_FLUX_ERR"} = "E";
    $suminfo{form}{"PN_2_FLUX"    } = "E";
    $suminfo{form}{"PN_2_FLUX_ERR"} = "E";
    $suminfo{form}{"PN_3_FLUX"    } = "E";
    $suminfo{form}{"PN_3_FLUX_ERR"} = "E";
    $suminfo{form}{"PN_4_FLUX"    } = "E";
    $suminfo{form}{"PN_4_FLUX_ERR"} = "E";
    $suminfo{form}{"PN_5_FLUX"    } = "E";
    $suminfo{form}{"PN_5_FLUX_ERR"} = "E";
    $suminfo{form}{"M1_FLUX"      } = "E";
    $suminfo{form}{"M1_FLUX_ERR"  } = "E";
    $suminfo{form}{"M1_1_FLUX"    } = "E";
    $suminfo{form}{"M1_1_FLUX_ERR"} = "E";
    $suminfo{form}{"M1_2_FLUX"    } = "E";
    $suminfo{form}{"M1_2_FLUX_ERR"} = "E";
    $suminfo{form}{"M1_3_FLUX"    } = "E";
    $suminfo{form}{"M1_3_FLUX_ERR"} = "E";
    $suminfo{form}{"M1_4_FLUX"    } = "E";
    $suminfo{form}{"M1_4_FLUX_ERR"} = "E";
    $suminfo{form}{"M1_5_FLUX"    } = "E";
    $suminfo{form}{"M1_5_FLUX_ERR"} = "E";
    $suminfo{form}{"M2_FLUX"      } = "E";
    $suminfo{form}{"M2_FLUX_ERR"  } = "E";
    $suminfo{form}{"M2_1_FLUX"    } = "E";
    $suminfo{form}{"M2_1_FLUX_ERR"} = "E";
    $suminfo{form}{"M2_2_FLUX"    } = "E";
    $suminfo{form}{"M2_2_FLUX_ERR"} = "E";
    $suminfo{form}{"M2_3_FLUX"    } = "E";
    $suminfo{form}{"M2_3_FLUX_ERR"} = "E";
    $suminfo{form}{"M2_4_FLUX"    } = "E";
    $suminfo{form}{"M2_4_FLUX_ERR"} = "E";
    $suminfo{form}{"M2_5_FLUX"    } = "E";
    $suminfo{form}{"M2_5_FLUX_ERR"} = "E";
    $suminfo{form}{"EP_RATE"      } = "E";
    $suminfo{form}{"EP_RATE_ERR"  } = "E";
    $suminfo{form}{"PN_RATE"      } = "E";
    $suminfo{form}{"PN_RATE_ERR"  } = "E";
    $suminfo{form}{"PN_1_RATE"    } = "E";
    $suminfo{form}{"PN_1_RATE_ERR"} = "E";
    $suminfo{form}{"PN_2_RATE"    } = "E";
    $suminfo{form}{"PN_2_RATE_ERR"} = "E";
    $suminfo{form}{"PN_3_RATE"    } = "E";
    $suminfo{form}{"PN_3_RATE_ERR"} = "E";
    $suminfo{form}{"PN_4_RATE"    } = "E";
    $suminfo{form}{"PN_4_RATE_ERR"} = "E";
    $suminfo{form}{"PN_5_RATE"    } = "E";
    $suminfo{form}{"PN_5_RATE_ERR"} = "E";
    $suminfo{form}{"M1_RATE"      } = "E";
    $suminfo{form}{"M1_RATE_ERR"  } = "E";
    $suminfo{form}{"M1_1_RATE"    } = "E";
    $suminfo{form}{"M1_1_RATE_ERR"} = "E";
    $suminfo{form}{"M1_2_RATE"    } = "E";
    $suminfo{form}{"M1_2_RATE_ERR"} = "E";
    $suminfo{form}{"M1_3_RATE"    } = "E";
    $suminfo{form}{"M1_3_RATE_ERR"} = "E";
    $suminfo{form}{"M1_4_RATE"    } = "E";
    $suminfo{form}{"M1_4_RATE_ERR"} = "E";
    $suminfo{form}{"M1_5_RATE"    } = "E";
    $suminfo{form}{"M1_5_RATE_ERR"} = "E";
    $suminfo{form}{"M2_RATE"      } = "E";
    $suminfo{form}{"M2_RATE_ERR"  } = "E";
    $suminfo{form}{"M2_1_RATE"    } = "E";
    $suminfo{form}{"M2_1_RATE_ERR"} = "E";
    $suminfo{form}{"M2_2_RATE"    } = "E";
    $suminfo{form}{"M2_2_RATE_ERR"} = "E";
    $suminfo{form}{"M2_3_RATE"    } = "E";
    $suminfo{form}{"M2_3_RATE_ERR"} = "E";
    $suminfo{form}{"M2_4_RATE"    } = "E";
    $suminfo{form}{"M2_4_RATE_ERR"} = "E";
    $suminfo{form}{"M2_5_RATE"    } = "E";
    $suminfo{form}{"M2_5_RATE_ERR"} = "E";
    $suminfo{form}{"EP_CTS"       } = "E";
    $suminfo{form}{"EP_CTS_ERR"   } = "E";
    $suminfo{form}{"PN_CTS"       } = "E";
    $suminfo{form}{"PN_CTS_ERR"   } = "E";
    $suminfo{form}{"M1_CTS"       } = "E";
    $suminfo{form}{"M1_CTS_ERR"   } = "E";
    $suminfo{form}{"M2_CTS"       } = "E";
    $suminfo{form}{"M2_CTS_ERR"   } = "E";
    $suminfo{form}{"EP_DET_ML"    } = "E";
    $suminfo{form}{"PN_DET_ML"    } = "E";
    $suminfo{form}{"PN_1_DET_ML"  } = "E";
    $suminfo{form}{"PN_2_DET_ML"  } = "E";
    $suminfo{form}{"PN_3_DET_ML"  } = "E";
    $suminfo{form}{"PN_4_DET_ML"  } = "E";
    $suminfo{form}{"PN_5_DET_ML"  } = "E";
    $suminfo{form}{"M1_DET_ML"  , } = "E";
    $suminfo{form}{"M1_1_DET_ML"  } = "E";
    $suminfo{form}{"M1_2_DET_ML"  } = "E";
    $suminfo{form}{"M1_3_DET_ML"  } = "E";
    $suminfo{form}{"M1_4_DET_ML"  } = "E";
    $suminfo{form}{"M1_5_DET_ML"  } = "E";
    $suminfo{form}{"M2_DET_ML"    } = "E";
    $suminfo{form}{"M2_1_DET_ML"  } = "E";
    $suminfo{form}{"M2_2_DET_ML"  } = "E";
    $suminfo{form}{"M2_3_DET_ML"  } = "E";
    $suminfo{form}{"M2_4_DET_ML"  } = "E";
    $suminfo{form}{"M2_5_DET_ML"  } = "E";
    $suminfo{form}{"EXTENT"       } = "E";
    $suminfo{form}{"EXTENT_ERR"   } = "E";
    $suminfo{form}{"EXTENT_ML"    } = "E";
    $suminfo{form}{"EP_HR1"       } = "E";
    $suminfo{form}{"EP_HR1_ERR"   } = "E";
    $suminfo{form}{"EP_HR2"       } = "E";
    $suminfo{form}{"EP_HR2_ERR"   } = "E";
    $suminfo{form}{"EP_HR3"       } = "E";
    $suminfo{form}{"EP_HR3_ERR"   } = "E";
    $suminfo{form}{"EP_HR4"       } = "E";
    $suminfo{form}{"EP_HR4_ERR"   } = "E";
    $suminfo{form}{"PN_HR1"       } = "E";
    $suminfo{form}{"PN_HR1_ERR"   } = "E";
    $suminfo{form}{"PN_HR2"       } = "E";
    $suminfo{form}{"PN_HR2_ERR"   } = "E";
    $suminfo{form}{"PN_HR3"       } = "E";
    $suminfo{form}{"PN_HR3_ERR"   } = "E";
    $suminfo{form}{"PN_HR4"       } = "E";
    $suminfo{form}{"PN_HR4_ERR"   } = "E";
    $suminfo{form}{"M1_HR1"       } = "E";
    $suminfo{form}{"M1_HR1_ERR"   } = "E";
    $suminfo{form}{"M1_HR2"       } = "E";
    $suminfo{form}{"M1_HR2_ERR"   } = "E";
    $suminfo{form}{"M1_HR3"       } = "E";
    $suminfo{form}{"M1_HR3_ERR"   } = "E";
    $suminfo{form}{"M1_HR4"       } = "E";
    $suminfo{form}{"M1_HR4_ERR"   } = "E";
    $suminfo{form}{"M2_HR1"       } = "E";
    $suminfo{form}{"M2_HR1_ERR"   } = "E";
    $suminfo{form}{"M2_HR2"       } = "E";
    $suminfo{form}{"M2_HR2_ERR"   } = "E";
    $suminfo{form}{"M2_HR3"       } = "E";
    $suminfo{form}{"M2_HR3_ERR"   } = "E";
    $suminfo{form}{"M2_HR4"       } = "E";
    $suminfo{form}{"M2_HR4_ERR"   } = "E";
    $suminfo{form}{"PN_EXP"       } = "E";
    $suminfo{form}{"PN_1_EXP"     } = "E";
    $suminfo{form}{"PN_2_EXP"     } = "E";
    $suminfo{form}{"PN_3_EXP"     } = "E";
    $suminfo{form}{"PN_4_EXP"     } = "E";
    $suminfo{form}{"PN_5_EXP"     } = "E";
    $suminfo{form}{"M1_EXP"       } = "E";
    $suminfo{form}{"M1_1_EXP"     } = "E";
    $suminfo{form}{"M1_2_EXP"     } = "E";
    $suminfo{form}{"M1_3_EXP"     } = "E";
    $suminfo{form}{"M1_4_EXP"     } = "E";
    $suminfo{form}{"M1_5_EXP"     } = "E";
    $suminfo{form}{"M2_EXP"       } = "E";
    $suminfo{form}{"M2_1_EXP"     } = "E";
    $suminfo{form}{"M2_2_EXP"     } = "E";
    $suminfo{form}{"M2_3_EXP"     } = "E";
    $suminfo{form}{"M2_4_EXP"     } = "E";
    $suminfo{form}{"M2_5_EXP"     } = "E";
    $suminfo{form}{"PN_BG"        } = "E";
    $suminfo{form}{"PN_1_BG"      } = "E";
    $suminfo{form}{"PN_2_BG"      } = "E";
    $suminfo{form}{"PN_3_BG"      } = "E";
    $suminfo{form}{"PN_4_BG"      } = "E";
    $suminfo{form}{"PN_5_BG"      } = "E";
    $suminfo{form}{"M1_BG"        } = "E";
    $suminfo{form}{"M1_1_BG"      } = "E";
    $suminfo{form}{"M1_2_BG"      } = "E";
    $suminfo{form}{"M1_3_BG"      } = "E";
    $suminfo{form}{"M1_4_BG"      } = "E";
    $suminfo{form}{"M1_5_BG"      } = "E";
    $suminfo{form}{"M2_BG"        } = "E";
    $suminfo{form}{"M2_1_BG"      } = "E";
    $suminfo{form}{"M2_2_BG"      } = "E";
    $suminfo{form}{"M2_3_BG"      } = "E";
    $suminfo{form}{"M2_4_BG"      } = "E";
    $suminfo{form}{"M2_5_BG"      } = "E";
    $suminfo{form}{"EP_ONTIME"    } = "E";
    $suminfo{form}{"PN_ONTIME"    } = "E";
    $suminfo{form}{"M1_ONTIME"    } = "E";
    $suminfo{form}{"M2_ONTIME"    } = "E";
    $suminfo{form}{"PN_PILEUP"    } = "E";
    $suminfo{form}{"M1_PILEUP"    } = "E";
    $suminfo{form}{"M2_PILEUP"    } = "E";
    $suminfo{form}{"PN_MASKFRAC"  } = "E";
    $suminfo{form}{"M1_MASKFRAC"  } = "E";
    $suminfo{form}{"M2_MASKFRAC"  } = "E";
    $suminfo{form}{"DIST_REF"     } = "E";
    $suminfo{form}{"EP_OFFAX"     } = "E";
    $suminfo{form}{"PN_OFFAX"     } = "E";
    $suminfo{form}{"M1_OFFAX"     } = "E";
    $suminfo{form}{"M2_OFFAX"     } = "E";
    $suminfo{form}{"PN_1_VIG"     } = "E";
    $suminfo{form}{"PN_2_VIG"     } = "E";
    $suminfo{form}{"PN_3_VIG"     } = "E";
    $suminfo{form}{"PN_4_VIG"     } = "E";
    $suminfo{form}{"PN_5_VIG"     } = "E";
    $suminfo{form}{"M1_1_VIG"     } = "E";
    $suminfo{form}{"M1_2_VIG"     } = "E";
    $suminfo{form}{"M1_3_VIG"     } = "E";
    $suminfo{form}{"M1_4_VIG"     } = "E";
    $suminfo{form}{"M1_5_VIG"     } = "E";
    $suminfo{form}{"M2_1_VIG"     } = "E";
    $suminfo{form}{"M2_2_VIG"     } = "E";
    $suminfo{form}{"M2_3_VIG"     } = "E";
    $suminfo{form}{"M2_4_VIG"     } = "E";
    $suminfo{form}{"M2_5_VIG"     } = "E";
    $suminfo{form}{"STACK_FLAG"   } = "I";
    $suminfo{form}{"EP_FLAG"      } = "9A";      # 3XMM: 12A
    $suminfo{form}{"PN_FLAG"      } = "9A";      #   - " -
    $suminfo{form}{"M1_FLAG"      } = "9A";      #   - " -
    $suminfo{form}{"M2_FLAG"      } = "9A";      #   - " -
    $suminfo{form}{"OVERLAP"      } = "L";
    $suminfo{form}{"VAR_CHI2"     } = "E";
    $suminfo{form}{"VAR_CHI2_1"   } = "E";
    $suminfo{form}{"VAR_CHI2_2"   } = "E";
    $suminfo{form}{"VAR_CHI2_3"   } = "E";
    $suminfo{form}{"VAR_CHI2_4"   } = "E";
    $suminfo{form}{"VAR_CHI2_5"   } = "E";
    $suminfo{form}{"VAR_PROB"     } = "D";
    $suminfo{form}{"VAR_PROB_1"   } = "D";
    $suminfo{form}{"VAR_PROB_2"   } = "D";
    $suminfo{form}{"VAR_PROB_3"   } = "D";
    $suminfo{form}{"VAR_PROB_4"   } = "D";
    $suminfo{form}{"VAR_PROB_5"   } = "D";
    $suminfo{form}{"FRATIO"       } = "E";
    $suminfo{form}{"FRATIO_1"     } = "E";
    $suminfo{form}{"FRATIO_2"     } = "E";
    $suminfo{form}{"FRATIO_3"     } = "E";
    $suminfo{form}{"FRATIO_4"     } = "E";
    $suminfo{form}{"FRATIO_5"     } = "E";
    $suminfo{form}{"FRATIO_ERR"   } = "E";
    $suminfo{form}{"FRATIO_1_ERR" } = "E";
    $suminfo{form}{"FRATIO_2_ERR" } = "E";
    $suminfo{form}{"FRATIO_3_ERR" } = "E";
    $suminfo{form}{"FRATIO_4_ERR" } = "E";
    $suminfo{form}{"FRATIO_5_ERR" } = "E";
    $suminfo{form}{"FLUXVAR"      } = "E";
    $suminfo{form}{"FLUXVAR_1"    } = "E";
    $suminfo{form}{"FLUXVAR_2"    } = "E";
    $suminfo{form}{"FLUXVAR_3"    } = "E";
    $suminfo{form}{"FLUXVAR_4"    } = "E";
    $suminfo{form}{"FLUXVAR_5"    } = "E";
    $suminfo{form}{"MJD_FIRST"    } = "D";
    $suminfo{form}{"MJD_LAST"     } = "D";
    $suminfo{form}{"REVOLUT"      } = "I";
    $suminfo{form}{"PA_PNT"       } = "E";
    $suminfo{form}{"PN_SUBMODE"   } = "23A";
    $suminfo{form}{"M1_SUBMODE"   } = "16A";
    $suminfo{form}{"M2_SUBMODE"   } = "16A";
    $suminfo{form}{"PN_FILTER"    } = "6A";
    $suminfo{form}{"M1_FILTER"    } = "6A";
    $suminfo{form}{"M2_FILTER"    } = "6A";

    # emllist only
    $suminfo{form}{"RAWX" } = "J";
    $suminfo{form}{"RAWY" } = "J";
    $suminfo{form}{"CCDNR"} = "J";
    $suminfo{form}{"EEF"  } = "E";

    # units
    $suminfo{unit}{"SRCID"        } = "";
    $suminfo{unit}{"OBS_ID"       } = "";
    $suminfo{unit}{"ML_ID_SRC"    } = "";
    $suminfo{unit}{"BOX_ID_SRC"   } = "";
    $suminfo{unit}{"N_OBS"        } = "";
    $suminfo{unit}{"N_CONTRIB"    } = "";
    $suminfo{unit}{"N_EXP"        } = "";
    $suminfo{unit}{"RA"           } = "deg";
    $suminfo{unit}{"DEC"          } = "deg";
    $suminfo{unit}{"RADEC_ERR"    } = "arcsec";
    $suminfo{unit}{"LII"          } = "deg";
    $suminfo{unit}{"BII"          } = "deg";
    $suminfo{unit}{"X_IMA"        } = "pixel";
    $suminfo{unit}{"X_IMA_ERR"    } = "pixel";
    $suminfo{unit}{"Y_IMA"        } = "pixel";
    $suminfo{unit}{"Y_IMA_ERR"    } = "pixel";
    $suminfo{unit}{"DIST_NN"      } = "arcsec";
    $suminfo{unit}{"N_BLEND"      } = "";
    $suminfo{unit}{"EP_FLUX"      } = "erg/cm**2/s";
    $suminfo{unit}{"EP_FLUX_ERR"  } = "erg/cm**2/s";
    $suminfo{unit}{"EP_1_FLUX"    } = "erg/cm**2/s";
    $suminfo{unit}{"EP_1_FLUX_ERR"} = "erg/cm**2/s";
    $suminfo{unit}{"EP_2_FLUX"    } = "erg/cm**2/s";
    $suminfo{unit}{"EP_2_FLUX_ERR"} = "erg/cm**2/s";
    $suminfo{unit}{"EP_3_FLUX"    } = "erg/cm**2/s";
    $suminfo{unit}{"EP_3_FLUX_ERR"} = "erg/cm**2/s";
    $suminfo{unit}{"EP_4_FLUX"    } = "erg/cm**2/s";
    $suminfo{unit}{"EP_4_FLUX_ERR"} = "erg/cm**2/s";
    $suminfo{unit}{"EP_5_FLUX"    } = "erg/cm**2/s";
    $suminfo{unit}{"EP_5_FLUX_ERR"} = "erg/cm**2/s";
    $suminfo{unit}{"PN_FLUX"      } = "erg/cm**2/s";
    $suminfo{unit}{"PN_FLUX_ERR"  } = "erg/cm**2/s";
    $suminfo{unit}{"PN_1_FLUX"    } = "erg/cm**2/s";
    $suminfo{unit}{"PN_1_FLUX_ERR"} = "erg/cm**2/s";
    $suminfo{unit}{"PN_2_FLUX"    } = "erg/cm**2/s";
    $suminfo{unit}{"PN_2_FLUX_ERR"} = "erg/cm**2/s";
    $suminfo{unit}{"PN_3_FLUX"    } = "erg/cm**2/s";
    $suminfo{unit}{"PN_3_FLUX_ERR"} = "erg/cm**2/s";
    $suminfo{unit}{"PN_4_FLUX"    } = "erg/cm**2/s";
    $suminfo{unit}{"PN_4_FLUX_ERR"} = "erg/cm**2/s";
    $suminfo{unit}{"PN_5_FLUX"    } = "erg/cm**2/s";
    $suminfo{unit}{"PN_5_FLUX_ERR"} = "erg/cm**2/s";
    $suminfo{unit}{"M1_FLUX"      } = "erg/cm**2/s";
    $suminfo{unit}{"M1_FLUX_ERR"  } = "erg/cm**2/s";
    $suminfo{unit}{"M1_1_FLUX"    } = "erg/cm**2/s";
    $suminfo{unit}{"M1_1_FLUX_ERR"} = "erg/cm**2/s";
    $suminfo{unit}{"M1_2_FLUX"    } = "erg/cm**2/s";
    $suminfo{unit}{"M1_2_FLUX_ERR"} = "erg/cm**2/s";
    $suminfo{unit}{"M1_3_FLUX"    } = "erg/cm**2/s";
    $suminfo{unit}{"M1_3_FLUX_ERR"} = "erg/cm**2/s";
    $suminfo{unit}{"M1_4_FLUX"    } = "erg/cm**2/s";
    $suminfo{unit}{"M1_4_FLUX_ERR"} = "erg/cm**2/s";
    $suminfo{unit}{"M1_5_FLUX"    } = "erg/cm**2/s";
    $suminfo{unit}{"M1_5_FLUX_ERR"} = "erg/cm**2/s";
    $suminfo{unit}{"M2_FLUX"      } = "erg/cm**2/s";
    $suminfo{unit}{"M2_FLUX_ERR"  } = "erg/cm**2/s";
    $suminfo{unit}{"M2_1_FLUX"    } = "erg/cm**2/s";
    $suminfo{unit}{"M2_1_FLUX_ERR"} = "erg/cm**2/s";
    $suminfo{unit}{"M2_2_FLUX"    } = "erg/cm**2/s";
    $suminfo{unit}{"M2_2_FLUX_ERR"} = "erg/cm**2/s";
    $suminfo{unit}{"M2_3_FLUX"    } = "erg/cm**2/s";
    $suminfo{unit}{"M2_3_FLUX_ERR"} = "erg/cm**2/s";
    $suminfo{unit}{"M2_4_FLUX"    } = "erg/cm**2/s";
    $suminfo{unit}{"M2_4_FLUX_ERR"} = "erg/cm**2/s";
    $suminfo{unit}{"M2_5_FLUX"    } = "erg/cm**2/s";
    $suminfo{unit}{"M2_5_FLUX_ERR"} = "erg/cm**2/s";
    $suminfo{unit}{"EP_RATE"      } = "counts/s";
    $suminfo{unit}{"EP_RATE_ERR"  } = "counts/s";
    $suminfo{unit}{"PN_RATE"      } = "counts/s";
    $suminfo{unit}{"PN_RATE_ERR"  } = "counts/s";
    $suminfo{unit}{"PN_1_RATE"    } = "counts/s";
    $suminfo{unit}{"PN_1_RATE_ERR"} = "counts/s";
    $suminfo{unit}{"PN_2_RATE"    } = "counts/s";
    $suminfo{unit}{"PN_2_RATE_ERR"} = "counts/s";
    $suminfo{unit}{"PN_3_RATE"    } = "counts/s";
    $suminfo{unit}{"PN_3_RATE_ERR"} = "counts/s";
    $suminfo{unit}{"PN_4_RATE"    } = "counts/s";
    $suminfo{unit}{"PN_4_RATE_ERR"} = "counts/s";
    $suminfo{unit}{"PN_5_RATE"    } = "counts/s";
    $suminfo{unit}{"PN_5_RATE_ERR"} = "counts/s";
    $suminfo{unit}{"M1_RATE"      } = "counts/s";
    $suminfo{unit}{"M1_RATE_ERR"  } = "counts/s";
    $suminfo{unit}{"M1_1_RATE"    } = "counts/s";
    $suminfo{unit}{"M1_1_RATE_ERR"} = "counts/s";
    $suminfo{unit}{"M1_2_RATE"    } = "counts/s";
    $suminfo{unit}{"M1_2_RATE_ERR"} = "counts/s";
    $suminfo{unit}{"M1_3_RATE"    } = "counts/s";
    $suminfo{unit}{"M1_3_RATE_ERR"} = "counts/s";
    $suminfo{unit}{"M1_4_RATE"    } = "counts/s";
    $suminfo{unit}{"M1_4_RATE_ERR"} = "counts/s";
    $suminfo{unit}{"M1_5_RATE"    } = "counts/s";
    $suminfo{unit}{"M1_5_RATE_ERR"} = "counts/s";
    $suminfo{unit}{"M2_RATE"      } = "counts/s";
    $suminfo{unit}{"M2_RATE_ERR"  } = "counts/s";
    $suminfo{unit}{"M2_1_RATE"    } = "counts/s";
    $suminfo{unit}{"M2_1_RATE_ERR"} = "counts/s";
    $suminfo{unit}{"M2_2_RATE"    } = "counts/s";
    $suminfo{unit}{"M2_2_RATE_ERR"} = "counts/s";
    $suminfo{unit}{"M2_3_RATE"    } = "counts/s";
    $suminfo{unit}{"M2_3_RATE_ERR"} = "counts/s";
    $suminfo{unit}{"M2_4_RATE"    } = "counts/s";
    $suminfo{unit}{"M2_4_RATE_ERR"} = "counts/s";
    $suminfo{unit}{"M2_5_RATE"    } = "counts/s";
    $suminfo{unit}{"M2_5_RATE_ERR"} = "counts/s";
    $suminfo{unit}{"EP_CTS"       } = "counts";
    $suminfo{unit}{"EP_CTS_ERR"   } = "counts";
    $suminfo{unit}{"PN_CTS"       } = "counts";
    $suminfo{unit}{"PN_CTS_ERR"   } = "counts";
    $suminfo{unit}{"M1_CTS"       } = "counts";
    $suminfo{unit}{"M1_CTS_ERR"   } = "counts";
    $suminfo{unit}{"M2_CTS"       } = "counts";
    $suminfo{unit}{"M2_CTS_ERR"   } = "counts";
    $suminfo{unit}{"EP_DET_ML"    } = "";
    $suminfo{unit}{"PN_DET_ML"    } = "";
    $suminfo{unit}{"PN_1_DET_ML"  } = "";
    $suminfo{unit}{"PN_2_DET_ML"  } = "";
    $suminfo{unit}{"PN_3_DET_ML"  } = "";
    $suminfo{unit}{"PN_4_DET_ML"  } = "";
    $suminfo{unit}{"PN_5_DET_ML"  } = "";
    $suminfo{unit}{"M1_DET_ML"  , } = "";
    $suminfo{unit}{"M1_1_DET_ML"  } = "";
    $suminfo{unit}{"M1_2_DET_ML"  } = "";
    $suminfo{unit}{"M1_3_DET_ML"  } = "";
    $suminfo{unit}{"M1_4_DET_ML"  } = "";
    $suminfo{unit}{"M1_5_DET_ML"  } = "";
    $suminfo{unit}{"M2_DET_ML"    } = "";
    $suminfo{unit}{"M2_1_DET_ML"  } = "";
    $suminfo{unit}{"M2_2_DET_ML"  } = "";
    $suminfo{unit}{"M2_3_DET_ML"  } = "";
    $suminfo{unit}{"M2_4_DET_ML"  } = "";
    $suminfo{unit}{"M2_5_DET_ML"  } = "";
    $suminfo{unit}{"EXTENT"       } = "arcsec";
    $suminfo{unit}{"EXTENT_ERR"   } = "arcsec";
    $suminfo{unit}{"EXTENT_ML"    } = "";
    $suminfo{unit}{"EP_HR1"       } = "";
    $suminfo{unit}{"EP_HR1_ERR"   } = "";
    $suminfo{unit}{"EP_HR2"       } = "";
    $suminfo{unit}{"EP_HR2_ERR"   } = "";
    $suminfo{unit}{"EP_HR3"       } = "";
    $suminfo{unit}{"EP_HR3_ERR"   } = "";
    $suminfo{unit}{"EP_HR4"       } = "";
    $suminfo{unit}{"EP_HR4_ERR"   } = "";
    $suminfo{unit}{"PN_HR1"       } = "";
    $suminfo{unit}{"PN_HR1_ERR"   } = "";
    $suminfo{unit}{"PN_HR2"       } = "";
    $suminfo{unit}{"PN_HR2_ERR"   } = "";
    $suminfo{unit}{"PN_HR3"       } = "";
    $suminfo{unit}{"PN_HR3_ERR"   } = "";
    $suminfo{unit}{"PN_HR4"       } = "";
    $suminfo{unit}{"PN_HR4_ERR"   } = "";
    $suminfo{unit}{"M1_HR1"       } = "";
    $suminfo{unit}{"M1_HR1_ERR"   } = "";
    $suminfo{unit}{"M1_HR2"       } = "";
    $suminfo{unit}{"M1_HR2_ERR"   } = "";
    $suminfo{unit}{"M1_HR3"       } = "";
    $suminfo{unit}{"M1_HR3_ERR"   } = "";
    $suminfo{unit}{"M1_HR4"       } = "";
    $suminfo{unit}{"M1_HR4_ERR"   } = "";
    $suminfo{unit}{"M2_HR1"       } = "";
    $suminfo{unit}{"M2_HR1_ERR"   } = "";
    $suminfo{unit}{"M2_HR2"       } = "";
    $suminfo{unit}{"M2_HR2_ERR"   } = "";
    $suminfo{unit}{"M2_HR3"       } = "";
    $suminfo{unit}{"M2_HR3_ERR"   } = "";
    $suminfo{unit}{"M2_HR4"       } = "";
    $suminfo{unit}{"M2_HR4_ERR"   } = "";
    $suminfo{unit}{"PN_EXP"       } = "s";
    $suminfo{unit}{"PN_1_EXP"     } = "s";
    $suminfo{unit}{"PN_2_EXP"     } = "s";
    $suminfo{unit}{"PN_3_EXP"     } = "s";
    $suminfo{unit}{"PN_4_EXP"     } = "s";
    $suminfo{unit}{"PN_5_EXP"     } = "s";
    $suminfo{unit}{"M1_EXP"       } = "s";
    $suminfo{unit}{"M1_1_EXP"     } = "s";
    $suminfo{unit}{"M1_2_EXP"     } = "s";
    $suminfo{unit}{"M1_3_EXP"     } = "s";
    $suminfo{unit}{"M1_4_EXP"     } = "s";
    $suminfo{unit}{"M1_5_EXP"     } = "s";
    $suminfo{unit}{"M2_EXP"       } = "s";
    $suminfo{unit}{"M2_1_EXP"     } = "s";
    $suminfo{unit}{"M2_2_EXP"     } = "s";
    $suminfo{unit}{"M2_3_EXP"     } = "s";
    $suminfo{unit}{"M2_4_EXP"     } = "s";
    $suminfo{unit}{"M2_5_EXP"     } = "s";
    $suminfo{unit}{"PN_BG"        } = "counts/pixel";
    $suminfo{unit}{"PN_1_BG"      } = "counts/pixel";
    $suminfo{unit}{"PN_2_BG"      } = "counts/pixel";
    $suminfo{unit}{"PN_3_BG"      } = "counts/pixel";
    $suminfo{unit}{"PN_4_BG"      } = "counts/pixel";
    $suminfo{unit}{"PN_5_BG"      } = "counts/pixel";
    $suminfo{unit}{"M1_BG"        } = "counts/pixel";
    $suminfo{unit}{"M1_1_BG"      } = "counts/pixel";
    $suminfo{unit}{"M1_2_BG"      } = "counts/pixel";
    $suminfo{unit}{"M1_3_BG"      } = "counts/pixel";
    $suminfo{unit}{"M1_4_BG"      } = "counts/pixel";
    $suminfo{unit}{"M1_5_BG"      } = "counts/pixel";
    $suminfo{unit}{"M2_BG"        } = "counts/pixel";
    $suminfo{unit}{"M2_1_BG"      } = "counts/pixel";
    $suminfo{unit}{"M2_2_BG"      } = "counts/pixel";
    $suminfo{unit}{"M2_3_BG"      } = "counts/pixel";
    $suminfo{unit}{"M2_4_BG"      } = "counts/pixel";
    $suminfo{unit}{"M2_5_BG"      } = "counts/pixel";
    $suminfo{unit}{"EP_ONTIME"    } = "s";
    $suminfo{unit}{"PN_ONTIME"    } = "s";
    $suminfo{unit}{"M1_ONTIME"    } = "s";
    $suminfo{unit}{"M2_ONTIME"    } = "s";
    $suminfo{unit}{"PN_PILEUP"    } = "";
    $suminfo{unit}{"M1_PILEUP"    } = "";
    $suminfo{unit}{"M2_PILEUP"    } = "";
    $suminfo{unit}{"PN_MASKFRAC"  } = "";
    $suminfo{unit}{"M1_MASKFRAC"  } = "";
    $suminfo{unit}{"M2_MASKFRAC"  } = "";
    $suminfo{unit}{"DIST_REF"     } = "arcmin";
    $suminfo{unit}{"EP_OFFAX"     } = "arcmin";
    $suminfo{unit}{"PN_OFFAX"     } = "arcmin";
    $suminfo{unit}{"M1_OFFAX"     } = "arcmin";
    $suminfo{unit}{"M2_OFFAX"     } = "arcmin";
    $suminfo{unit}{"PN_1_VIG"     } = "";
    $suminfo{unit}{"PN_2_VIG"     } = "";
    $suminfo{unit}{"PN_3_VIG"     } = "";
    $suminfo{unit}{"PN_4_VIG"     } = "";
    $suminfo{unit}{"PN_5_VIG"     } = "";
    $suminfo{unit}{"M1_1_VIG"     } = "";
    $suminfo{unit}{"M1_2_VIG"     } = "";
    $suminfo{unit}{"M1_3_VIG"     } = "";
    $suminfo{unit}{"M1_4_VIG"     } = "";
    $suminfo{unit}{"M1_5_VIG"     } = "";
    $suminfo{unit}{"M2_1_VIG"     } = "";
    $suminfo{unit}{"M2_2_VIG"     } = "";
    $suminfo{unit}{"M2_3_VIG"     } = "";
    $suminfo{unit}{"M2_4_VIG"     } = "";
    $suminfo{unit}{"M2_5_VIG"     } = "";
    $suminfo{unit}{"STACK_FLAG"   } = "";
    $suminfo{unit}{"EP_FLAG"      } = "";
    $suminfo{unit}{"PN_FLAG"      } = "";
    $suminfo{unit}{"M1_FLAG"      } = "";
    $suminfo{unit}{"M2_FLAG"      } = "";
    $suminfo{unit}{"OVERLAP"      } = "";
    $suminfo{unit}{"VAR_CHI2"     } = "";
    $suminfo{unit}{"VAR_CHI2_1"   } = "";
    $suminfo{unit}{"VAR_CHI2_2"   } = "";
    $suminfo{unit}{"VAR_CHI2_3"   } = "";
    $suminfo{unit}{"VAR_CHI2_4"   } = "";
    $suminfo{unit}{"VAR_CHI2_5"   } = "";
    $suminfo{unit}{"VAR_PROB"     } = "";
    $suminfo{unit}{"VAR_PROB_1"   } = "";
    $suminfo{unit}{"VAR_PROB_2"   } = "";
    $suminfo{unit}{"VAR_PROB_3"   } = "";
    $suminfo{unit}{"VAR_PROB_4"   } = "";
    $suminfo{unit}{"VAR_PROB_5"   } = "";
    $suminfo{unit}{"FRATIO"       } = "";
    $suminfo{unit}{"FRATIO_1"     } = "";
    $suminfo{unit}{"FRATIO_2"     } = "";
    $suminfo{unit}{"FRATIO_3"     } = "";
    $suminfo{unit}{"FRATIO_4"     } = "";
    $suminfo{unit}{"FRATIO_5"     } = "";
    $suminfo{unit}{"FRATIO_ERR"   } = "";
    $suminfo{unit}{"FRATIO_1_ERR" } = "";
    $suminfo{unit}{"FRATIO_2_ERR" } = "";
    $suminfo{unit}{"FRATIO_3_ERR" } = "";
    $suminfo{unit}{"FRATIO_4_ERR" } = "";
    $suminfo{unit}{"FRATIO_5_ERR" } = "";
    $suminfo{unit}{"FLUXVAR"      } = "";
    $suminfo{unit}{"FLUXVAR_1"    } = "";
    $suminfo{unit}{"FLUXVAR_2"    } = "";
    $suminfo{unit}{"FLUXVAR_3"    } = "";
    $suminfo{unit}{"FLUXVAR_4"    } = "";
    $suminfo{unit}{"FLUXVAR_5"    } = "";
    $suminfo{unit}{"MJD_FIRST"    } = "d";
    $suminfo{unit}{"MJD_LAST"     } = "d";
    $suminfo{unit}{"REVOLUT"      } = "orbit";
    $suminfo{unit}{"PA_PNT"       } = "deg";
    $suminfo{unit}{"PN_SUBMODE"   } = "";
    $suminfo{unit}{"M1_SUBMODE"   } = "";
    $suminfo{unit}{"M2_SUBMODE"   } = "";
    $suminfo{unit}{"PN_FILTER"    } = "";
    $suminfo{unit}{"M1_FILTER"    } = "";
    $suminfo{unit}{"M2_FILTER"    } = "";

    # comment rows
    $suminfo{comm}{"SRCID"        } = "Identifier of the source";
    $suminfo{comm}{"OBS_ID"       } = "XMM-Newton observation identification";
    $suminfo{comm}{"ML_ID_SRC"    } = "Identifier of the detection within the original emldetect list";
    $suminfo{comm}{"BOX_ID_SRC"   } = "Identifier of the detection within the original eboxdetect list";
    $suminfo{comm}{"N_OBS"        } = "Number of observations involved in the stack";
    $suminfo{comm}{"N_CONTRIB"    } = "Number of observations in which the source was fitted";
    $suminfo{comm}{"N_EXP"        } = "Number of instruments for which the source was fitted";
    $suminfo{comm}{"RA"           } = "Right ascension (J2000)";
    $suminfo{comm}{"DEC"          } = "Declination (J2000)";
    $suminfo{comm}{"RADEC_ERR"    } = "Square root of squared sum of 1-sigma errors in RA and DEC";
    $suminfo{comm}{"LII"          } = "Galactic longitude";
    $suminfo{comm}{"BII"          } = "Galactic latitude";
    $suminfo{comm}{"X_IMA"        } = "X image coordinate";
    $suminfo{comm}{"X_IMA_ERR"    } = "1-sigma error on X_IMA";
    $suminfo{comm}{"Y_IMA"        } = "Y image coordinate";
    $suminfo{comm}{"Y_IMA_ERR"    } = "1-sigma error on Y_IMA";
    $suminfo{comm}{"DIST_NN"      } = "Distance to the nearest neighbouring detection";
    $suminfo{comm}{"N_BLEND"      } = "Number of simultaneously fit sources";
    $suminfo{comm}{"EP_FLUX"      } = "All-EPIC flux";
    $suminfo{comm}{"EP_FLUX_ERR"  } = "1-sigma error on EP_FLUX";
    $suminfo{comm}{"EP_1_FLUX"    } = "Total flux in energy band 1";
    $suminfo{comm}{"EP_1_FLUX_ERR"} = "1-sigma error on EP_1_FLUX";
    $suminfo{comm}{"EP_2_FLUX"    } = "Total flux in energy band 2";
    $suminfo{comm}{"EP_2_FLUX_ERR"} = "1-sigma error on EP_2_FLUX";
    $suminfo{comm}{"EP_3_FLUX"    } = "Total flux in energy band 3";
    $suminfo{comm}{"EP_3_FLUX_ERR"} = "1-sigma error on EP_3_FLUX";
    $suminfo{comm}{"EP_4_FLUX"    } = "Total flux in energy band 4";
    $suminfo{comm}{"EP_4_FLUX_ERR"} = "1-sigma error on EP_4_FLUX";
    $suminfo{comm}{"EP_5_FLUX"    } = "Total flux in energy band 5";
    $suminfo{comm}{"EP_5_FLUX_ERR"} = "1-sigma error on EP_5_FLUX";
    $suminfo{comm}{"PN_FLUX"      } = "Total EPIC/pn flux";
    $suminfo{comm}{"PN_FLUX_ERR"  } = "1-sigma error on PN_FLUX";
    $suminfo{comm}{"PN_1_FLUX"    } = "EPIC/pn flux in energy band 1";
    $suminfo{comm}{"PN_1_FLUX_ERR"} = "1-sigma error on PN_1_FLUX";
    $suminfo{comm}{"PN_2_FLUX"    } = "EPIC/pn flux in energy band 2";
    $suminfo{comm}{"PN_2_FLUX_ERR"} = "1-sigma error on PN_2_FLUX";
    $suminfo{comm}{"PN_3_FLUX"    } = "EPIC/pn flux in energy band 3";
    $suminfo{comm}{"PN_3_FLUX_ERR"} = "1-sigma error on PN_3_FLUX";
    $suminfo{comm}{"PN_4_FLUX"    } = "EPIC/pn flux in energy band 4";
    $suminfo{comm}{"PN_4_FLUX_ERR"} = "1-sigma error on PN_4_FLUX";
    $suminfo{comm}{"PN_5_FLUX"    } = "EPIC/pn flux in energy band 5";
    $suminfo{comm}{"PN_5_FLUX_ERR"} = "1-sigma error on PN_5_FLUX";
    $suminfo{comm}{"M1_FLUX"      } = "Total EPIC/MOS1 flux";
    $suminfo{comm}{"M1_FLUX_ERR"  } = "1-sigma error on M1_FLUX";
    $suminfo{comm}{"M1_1_FLUX"    } = "EPIC/MOS1 flux in energy band 1";
    $suminfo{comm}{"M1_1_FLUX_ERR"} = "1-sigma error on M1_1_FLUX";
    $suminfo{comm}{"M1_2_FLUX"    } = "EPIC/MOS1 flux in energy band 2";
    $suminfo{comm}{"M1_2_FLUX_ERR"} = "1-sigma error on M1_2_FLUX";
    $suminfo{comm}{"M1_3_FLUX"    } = "EPIC/MOS1 flux in energy band 3";
    $suminfo{comm}{"M1_3_FLUX_ERR"} = "1-sigma error on M1_3_FLUX";
    $suminfo{comm}{"M1_4_FLUX"    } = "EPIC/MOS1 flux in energy band 4";
    $suminfo{comm}{"M1_4_FLUX_ERR"} = "1-sigma error on M1_4_FLUX";
    $suminfo{comm}{"M1_5_FLUX"    } = "EPIC/MOS1 flux in energy band 5";
    $suminfo{comm}{"M1_5_FLUX_ERR"} = "1-sigma error on M1_5_FLUX";
    $suminfo{comm}{"M2_FLUX"      } = "Total EPIC/MOS2 flux";
    $suminfo{comm}{"M2_FLUX_ERR"  } = "1-sigma error on M2_FLUX";
    $suminfo{comm}{"M2_1_FLUX"    } = "EPIC/MOS2 flux in energy band 1";
    $suminfo{comm}{"M2_1_FLUX_ERR"} = "1-sigma error on M2_1_FLUX";
    $suminfo{comm}{"M2_2_FLUX"    } = "EPIC/MOS2 flux in energy band 2";
    $suminfo{comm}{"M2_2_FLUX_ERR"} = "1-sigma error on M2_2_FLUX";
    $suminfo{comm}{"M2_3_FLUX"    } = "EPIC/MOS2 flux in energy band 3";
    $suminfo{comm}{"M2_3_FLUX_ERR"} = "1-sigma error on M2_3_FLUX";
    $suminfo{comm}{"M2_4_FLUX"    } = "EPIC/MOS2 flux in energy band 4";
    $suminfo{comm}{"M2_4_FLUX_ERR"} = "1-sigma error on M2_4_FLUX";
    $suminfo{comm}{"M2_5_FLUX"    } = "EPIC/MOS2 flux in energy band 5";
    $suminfo{comm}{"M2_5_FLUX_ERR"} = "1-sigma error on M2_5_FLUX";
    $suminfo{comm}{"EP_RATE"      } = "All-EPIC count rate";
    $suminfo{comm}{"EP_RATE_ERR"  } = "1-sigma error on EP_RATE";
    $suminfo{comm}{"PN_RATE"      } = "Total EPIC/pn count rate";
    $suminfo{comm}{"PN_RATE_ERR"  } = "1-sigma error on PN_RATE";
    $suminfo{comm}{"PN_1_RATE"    } = "EPIC/pn count rate in energy band 1";
    $suminfo{comm}{"PN_1_RATE_ERR"} = "1-sigma error on PN_1_RATE";
    $suminfo{comm}{"PN_2_RATE"    } = "EPIC/pn count rate in energy band 2";
    $suminfo{comm}{"PN_2_RATE_ERR"} = "1-sigma error on PN_2_RATE";
    $suminfo{comm}{"PN_3_RATE"    } = "EPIC/pn count rate in energy band 3";
    $suminfo{comm}{"PN_3_RATE_ERR"} = "1-sigma error on PN_3_RATE";
    $suminfo{comm}{"PN_4_RATE"    } = "EPIC/pn count rate in energy band 4";
    $suminfo{comm}{"PN_4_RATE_ERR"} = "1-sigma error on PN_4_RATE";
    $suminfo{comm}{"PN_5_RATE"    } = "EPIC/pn count rate in energy band 5";
    $suminfo{comm}{"PN_5_RATE_ERR"} = "1-sigma error on PN_5_RATE";
    $suminfo{comm}{"M1_RATE"      } = "Total EPIC/MOS1 count rate";
    $suminfo{comm}{"M1_RATE_ERR"  } = "1-sigma error on M1_RATE";
    $suminfo{comm}{"M1_1_RATE"    } = "EPIC/MOS1 count rate in energy band 1";
    $suminfo{comm}{"M1_1_RATE_ERR"} = "1-sigma error on M1_1_RATE";
    $suminfo{comm}{"M1_2_RATE"    } = "EPIC/MOS1 count rate in energy band 2";
    $suminfo{comm}{"M1_2_RATE_ERR"} = "1-sigma error on M1_2_RATE";
    $suminfo{comm}{"M1_3_RATE"    } = "EPIC/MOS1 count rate in energy band 3";
    $suminfo{comm}{"M1_3_RATE_ERR"} = "1-sigma error on M1_3_RATE";
    $suminfo{comm}{"M1_4_RATE"    } = "EPIC/MOS1 count rate in energy band 4";
    $suminfo{comm}{"M1_4_RATE_ERR"} = "1-sigma error on M1_4_RATE";
    $suminfo{comm}{"M1_5_RATE"    } = "EPIC/MOS1 count rate in energy band 5";
    $suminfo{comm}{"M1_5_RATE_ERR"} = "1-sigma error on M1_5_RATE";
    $suminfo{comm}{"M2_RATE"      } = "Total EPIC/MOS2 count rate";
    $suminfo{comm}{"M2_RATE_ERR"  } = "1-sigma error on M2_RATE";
    $suminfo{comm}{"M2_1_RATE"    } = "EPIC/MOS2 count rate in energy band 1";
    $suminfo{comm}{"M2_1_RATE_ERR"} = "1-sigma error on M2_1_RATE";
    $suminfo{comm}{"M2_2_RATE"    } = "EPIC/MOS2 count rate in energy band 2";
    $suminfo{comm}{"M2_2_RATE_ERR"} = "1-sigma error on M2_2_RATE";
    $suminfo{comm}{"M2_3_RATE"    } = "EPIC/MOS2 count rate in energy band 3";
    $suminfo{comm}{"M2_3_RATE_ERR"} = "1-sigma error on M2_3_RATE";
    $suminfo{comm}{"M2_4_RATE"    } = "EPIC/MOS2 count rate in energy band 4";
    $suminfo{comm}{"M2_4_RATE_ERR"} = "1-sigma error on M2_4_RATE";
    $suminfo{comm}{"M2_5_RATE"    } = "EPIC/MOS2 count rate in energy band 5";
    $suminfo{comm}{"M2_5_RATE_ERR"} = "1-sigma error on M2_5_RATE";
    $suminfo{comm}{"EP_CTS"       } = "All-EPIC number of counts";
    $suminfo{comm}{"EP_CTS_ERR"   } = "1-sigma error on EP_CTS";
    $suminfo{comm}{"PN_CTS"       } = "EPIC/pn number of counts";
    $suminfo{comm}{"PN_CTS_ERR"   } = "1-sigma error on PN_CTS";
    $suminfo{comm}{"M1_CTS"       } = "EPIC/MOS1 number of counts";
    $suminfo{comm}{"M1_CTS_ERR"   } = "1-sigma error on M1_CTS";
    $suminfo{comm}{"M2_CTS"       } = "EPIC/MOS2 number of counts";
    $suminfo{comm}{"M2_CTS_ERR"   } = "1-sigma error on M2_CTS";
    $suminfo{comm}{"EP_DET_ML"    } = "All-EPIC equivalent maximum detection likelihood";
    $suminfo{comm}{"PN_DET_ML"    } = "EPIC/pn equivalent maximum detection likelihood";
    $suminfo{comm}{"PN_1_DET_ML"  } = "EPIC/pn equ. maximum detection likelihood in energy band 1";
    $suminfo{comm}{"PN_2_DET_ML"  } = "EPIC/pn equ. maximum detection likelihood in energy band 2";
    $suminfo{comm}{"PN_3_DET_ML"  } = "EPIC/pn equ. maximum detection likelihood in energy band 3";
    $suminfo{comm}{"PN_4_DET_ML"  } = "EPIC/pn equ. maximum detection likelihood in energy band 4";
    $suminfo{comm}{"PN_5_DET_ML"  } = "EPIC/pn equ. maximum detection likelihood in energy band 5";
    $suminfo{comm}{"M1_DET_ML"  , } = "EPIC/MOS1 equivalent maximum detection likelihood";
    $suminfo{comm}{"M1_1_DET_ML"  } = "EPIC/MOS1 equ. maximum detection likelihood in energy band 1";
    $suminfo{comm}{"M1_2_DET_ML"  } = "EPIC/MOS1 equ. maximum detection likelihood in energy band 2";
    $suminfo{comm}{"M1_3_DET_ML"  } = "EPIC/MOS1 equ. maximum detection likelihood in energy band 3";
    $suminfo{comm}{"M1_4_DET_ML"  } = "EPIC/MOS1 equ. maximum detection likelihood in energy band 4";
    $suminfo{comm}{"M1_5_DET_ML"  } = "EPIC/MOS1 equ. maximum detection likelihood in energy band 5";
    $suminfo{comm}{"M2_DET_ML"    } = "EPIC/MOS2 equivalent maximum detection likelihood";
    $suminfo{comm}{"M2_1_DET_ML"  } = "EPIC/MOS2 equ. maximum detection likelihood in energy band 1";
    $suminfo{comm}{"M2_2_DET_ML"  } = "EPIC/MOS2 equ. maximum detection likelihood in energy band 2";
    $suminfo{comm}{"M2_3_DET_ML"  } = "EPIC/MOS2 equ. maximum detection likelihood in energy band 3";
    $suminfo{comm}{"M2_4_DET_ML"  } = "EPIC/MOS2 equ. maximum detection likelihood in energy band 4";
    $suminfo{comm}{"M2_5_DET_ML"  } = "EPIC/MOS2 equ. maximum detection likelihood in energy band 5";
    $suminfo{comm}{"EXTENT"       } = "Extent radius";
    $suminfo{comm}{"EXTENT_ERR"   } = "1-sigma error on EXTENT";
    $suminfo{comm}{"EXTENT_ML"    } = "Likelihood of the detection being extended";
    $suminfo{comm}{"EP_HR1"       } = "All-EPIC hardness ratio of energy bands 1 and 2";
    $suminfo{comm}{"EP_HR1_ERR"   } = "1-sigma error on EP_HR1";
    $suminfo{comm}{"EP_HR2"       } = "All-EPIC hardness ratio of energy bands 2 and 3";
    $suminfo{comm}{"EP_HR2_ERR"   } = "1-sigma error on EP_HR2";
    $suminfo{comm}{"EP_HR3"       } = "All-EPIC hardness ratio of energy bands 3 and 4";
    $suminfo{comm}{"EP_HR3_ERR"   } = "1-sigma error on EP_HR3";
    $suminfo{comm}{"EP_HR4"       } = "All-EPIC hardness ratio of energy bands 4 and 5";
    $suminfo{comm}{"EP_HR4_ERR"   } = "1-sigma error on EP_HR4";
    $suminfo{comm}{"PN_HR1"       } = "EPIC/pn hardness ratio of energy bands 1 and 2";
    $suminfo{comm}{"PN_HR1_ERR"   } = "1-sigma error on PN_HR1";
    $suminfo{comm}{"PN_HR2"       } = "EPIC/pn hardness ratio of energy bands 2 and 3";
    $suminfo{comm}{"PN_HR2_ERR"   } = "1-sigma error on PN_HR2";
    $suminfo{comm}{"PN_HR3"       } = "EPIC/pn hardness ratio of energy bands 3 and 4";
    $suminfo{comm}{"PN_HR3_ERR"   } = "1-sigma error on PN_HR3";
    $suminfo{comm}{"PN_HR4"       } = "EPIC/pn hardness ratio of energy bands 4 and 5";
    $suminfo{comm}{"PN_HR4_ERR"   } = "1-sigma error on PN_HR4";
    $suminfo{comm}{"M1_HR1"       } = "EPIC/MOS1 hardness ratio of energy bands 1 and 2";
    $suminfo{comm}{"M1_HR1_ERR"   } = "1-sigma error on M1_HR1";
    $suminfo{comm}{"M1_HR2"       } = "EPIC/MOS1 hardness ratio of energy bands 2 and 3";
    $suminfo{comm}{"M1_HR2_ERR"   } = "1-sigma error on M1_HR2";
    $suminfo{comm}{"M1_HR3"       } = "EPIC/MOS1 hardness ratio of energy bands 3 and 4";
    $suminfo{comm}{"M1_HR3_ERR"   } = "1-sigma error on M1_HR3";
    $suminfo{comm}{"M1_HR4"       } = "EPIC/MOS1 hardness ratio of energy bands 4 and 5";
    $suminfo{comm}{"M1_HR4_ERR"   } = "1-sigma error on M1_HR4";
    $suminfo{comm}{"M2_HR1"       } = "EPIC/MOS2 hardness ratio of energy bands 1 and 2";
    $suminfo{comm}{"M2_HR1_ERR"   } = "1-sigma error on M2_HR1";
    $suminfo{comm}{"M2_HR2"       } = "EPIC/MOS2 hardness ratio of energy bands 2 and 3";
    $suminfo{comm}{"M2_HR2_ERR"   } = "1-sigma error on M2_HR2";
    $suminfo{comm}{"M2_HR3"       } = "EPIC/MOS2 hardness ratio of energy bands 3 and 4";
    $suminfo{comm}{"M2_HR3_ERR"   } = "1-sigma error on M2_HR3";
    $suminfo{comm}{"M2_HR4"       } = "EPIC/MOS2 hardness ratio of energy bands 4 and 5";
    $suminfo{comm}{"M2_HR4_ERR"   } = "1-sigma error on M2_HR4";
    $suminfo{comm}{"PN_EXP"       } = "PSF-weighted exposure of the detection in EPIC/pn";
    $suminfo{comm}{"PN_1_EXP"     } = "PSF-weighted exposure of the detection in energy band 1 of EPIC/pn";
    $suminfo{comm}{"PN_2_EXP"     } = "PSF-weighted exposure of the detection in energy band 2 of EPIC/pn";
    $suminfo{comm}{"PN_3_EXP"     } = "PSF-weighted exposure of the detection in energy band 3 of EPIC/pn";
    $suminfo{comm}{"PN_4_EXP"     } = "PSF-weighted exposure of the detection in energy band 4 of EPIC/pn";
    $suminfo{comm}{"PN_5_EXP"     } = "PSF-weighted exposure of the detection in energy band 5 of EPIC/pn";
    $suminfo{comm}{"M1_EXP"       } = "PSF-weighted exposure of the detection in EPIC/MOS1";
    $suminfo{comm}{"M1_1_EXP"     } = "PSF-weighted exposure of the detection in energy band 1 of EPIC/MOS1";
    $suminfo{comm}{"M1_2_EXP"     } = "PSF-weighted exposure of the detection in energy band 2 of EPIC/MOS1";
    $suminfo{comm}{"M1_3_EXP"     } = "PSF-weighted exposure of the detection in energy band 3 of EPIC/MOS1";
    $suminfo{comm}{"M1_4_EXP"     } = "PSF-weighted exposure of the detection in energy band 4 of EPIC/MOS1";
    $suminfo{comm}{"M1_5_EXP"     } = "PSF-weighted exposure of the detection in energy band 5 of EPIC/MOS1";
    $suminfo{comm}{"M2_EXP"       } = "PSF-weighted exposure of the detection in EPIC/MOS2";
    $suminfo{comm}{"M2_1_EXP"     } = "PSF-weighted exposure of the detection in energy band 1 of EPIC/MOS2";
    $suminfo{comm}{"M2_2_EXP"     } = "PSF-weighted exposure of the detection in energy band 2 of EPIC/MOS2";
    $suminfo{comm}{"M2_3_EXP"     } = "PSF-weighted exposure of the detection in energy band 3 of EPIC/MOS2";
    $suminfo{comm}{"M2_4_EXP"     } = "PSF-weighted exposure of the detection in energy band 4 of EPIC/MOS2";
    $suminfo{comm}{"M2_5_EXP"     } = "PSF-weighted exposure of the detection in energy band 5 of EPIC/MOS2";
    $suminfo{comm}{"PN_BG"        } = "EPIC/pn background map at detection position";
    $suminfo{comm}{"PN_1_BG"      } = "EPIC/pn background map in energy band 1 at detection position";
    $suminfo{comm}{"PN_2_BG"      } = "EPIC/pn background map in energy band 2 at detection position";
    $suminfo{comm}{"PN_3_BG"      } = "EPIC/pn background map in energy band 3 at detection position";
    $suminfo{comm}{"PN_4_BG"      } = "EPIC/pn background map in energy band 4 at detection position";
    $suminfo{comm}{"PN_5_BG"      } = "EPIC/pn background map in energy band 5 at detection position";
    $suminfo{comm}{"M1_BG"        } = "EPIC/MOS1 background map at detection position";
    $suminfo{comm}{"M1_1_BG"      } = "EPIC/MOS1 background map in energy band 1 at detection position";
    $suminfo{comm}{"M1_2_BG"      } = "EPIC/MOS1 background map in energy band 2 at detection position";
    $suminfo{comm}{"M1_3_BG"      } = "EPIC/MOS1 background map in energy band 3 at detection position";
    $suminfo{comm}{"M1_4_BG"      } = "EPIC/MOS1 background map in energy band 4 at detection position";
    $suminfo{comm}{"M1_5_BG"      } = "EPIC/MOS1 background map in energy band 5 at detection position";
    $suminfo{comm}{"M2_BG"        } = "EPIC/MOS2 background map at detection position";
    $suminfo{comm}{"M2_1_BG"      } = "EPIC/MOS2 background map in energy band 1 at detection position";
    $suminfo{comm}{"M2_2_BG"      } = "EPIC/MOS2 background map in energy band 2 at detection position";
    $suminfo{comm}{"M2_3_BG"      } = "EPIC/MOS2 background map in energy band 3 at detection position";
    $suminfo{comm}{"M2_4_BG"      } = "EPIC/MOS2 background map in energy band 4 at detection position";
    $suminfo{comm}{"M2_5_BG"      } = "EPIC/MOS2 background map in energy band 5 at detection position";
    $suminfo{comm}{"EP_ONTIME"    } = "Total good exposure time all-EPIC";
    $suminfo{comm}{"PN_ONTIME"    } = "Total good exposure time in EPIC/pn";
    $suminfo{comm}{"M1_ONTIME"    } = "Total good exposure time in EPIC/MOS1";
    $suminfo{comm}{"M2_ONTIME"    } = "Total good exposure time in EPIC/MOS2";
    $suminfo{comm}{"PN_PILEUP"    } = "Estimate of the pile-up level in EPIC/pn";
    $suminfo{comm}{"M1_PILEUP"    } = "Estimate of the pile-up level in EPIC/MOS1";
    $suminfo{comm}{"M2_PILEUP"    } = "Estimate of the pile-up level in EPIC/MOS2";
    $suminfo{comm}{"PN_MASKFRAC"  } = "PSF-weighted detector coverage in EPIC/pn";
    $suminfo{comm}{"M1_MASKFRAC"  } = "PSF-weighted detector coverage in EPIC/MOS1";
    $suminfo{comm}{"M2_MASKFRAC"  } = "PSF-weighted detector coverage in EPIC/MOS2";
    $suminfo{comm}{"DIST_REF"     } = "Distance to the reference coordinates of the field";
    $suminfo{comm}{"EP_OFFAX"     } = "Offaxis angle between source position and aim point all-EPIC";
    $suminfo{comm}{"PN_OFFAX"     } = "Offaxis angle between source position and aim point in EPIC/pn";
    $suminfo{comm}{"M1_OFFAX"     } = "Offaxis angle between source position and aim point in EPIC/MOS1";
    $suminfo{comm}{"M2_OFFAX"     } = "Offaxis angle between source position and aim point in EPIC/MOS2";
    $suminfo{comm}{"PN_1_VIG"     } = "EPIC/pn vignetting factor in energy band 1 for the observation";
    $suminfo{comm}{"PN_2_VIG"     } = "EPIC/pn vignetting factor in energy band 2 for the observation";
    $suminfo{comm}{"PN_3_VIG"     } = "EPIC/pn vignetting factor in energy band 3 for the observation";
    $suminfo{comm}{"PN_4_VIG"     } = "EPIC/pn vignetting factor in energy band 4 for the observation";
    $suminfo{comm}{"PN_5_VIG"     } = "EPIC/pn vignetting factor in energy band 5 for the observation";
    $suminfo{comm}{"M1_1_VIG"     } = "EPIC/MOS1 vignetting factor in energy band 1 for the observation";
    $suminfo{comm}{"M1_2_VIG"     } = "EPIC/MOS1 vignetting factor in energy band 2 for the observation";
    $suminfo{comm}{"M1_3_VIG"     } = "EPIC/MOS1 vignetting factor in energy band 3 for the observation";
    $suminfo{comm}{"M1_4_VIG"     } = "EPIC/MOS1 vignetting factor in energy band 4 for the observation";
    $suminfo{comm}{"M1_5_VIG"     } = "EPIC/MOS1 vignetting factor in energy band 5 for the observation";
    $suminfo{comm}{"M2_1_VIG"     } = "EPIC/MOS2 vignetting factor in energy band 1 for the observation";
    $suminfo{comm}{"M2_2_VIG"     } = "EPIC/MOS2 vignetting factor in energy band 2 for the observation";
    $suminfo{comm}{"M2_3_VIG"     } = "EPIC/MOS2 vignetting factor in energy band 3 for the observation";
    $suminfo{comm}{"M2_4_VIG"     } = "EPIC/MOS2 vignetting factor in energy band 4 for the observation";
    $suminfo{comm}{"M2_5_VIG"     } = "EPIC/MOS2 vignetting factor in energy band 5 for the observation";
    $suminfo{comm}{"STACK_FLAG"   } = "Integer representation of the stack detection flags";
    $suminfo{comm}{"EP_FLAG"      } = "All-EPIC detection flags";
    $suminfo{comm}{"PN_FLAG"      } = "EPIC/pn detection flags";
    $suminfo{comm}{"M1_FLAG"      } = "EPIC/MOS1 detection flags";
    $suminfo{comm}{"M2_FLAG"      } = "EPIC/MOS2 detection flags";
    $suminfo{comm}{"OVERLAP"      } = "True, if N_CONTRIB>=2 at the source position";
    $suminfo{comm}{"VAR_CHI2"     } = "Reduced chi square of EPIC inter-observation variability";
    $suminfo{comm}{"VAR_CHI2_1"   } = "Reduced chi square of EPIC band 1 inter-observation variability";
    $suminfo{comm}{"VAR_CHI2_2"   } = "Reduced chi square of EPIC band 2 inter-observation variability";
    $suminfo{comm}{"VAR_CHI2_3"   } = "Reduced chi square of EPIC band 3 inter-observation variability";
    $suminfo{comm}{"VAR_CHI2_4"   } = "Reduced chi square of EPIC band 4 inter-observation variability";
    $suminfo{comm}{"VAR_CHI2_5"   } = "Reduced chi square of EPIC band 5 inter-observation variability";
    $suminfo{comm}{"VAR_PROB"     } = "Probability that the EPIC flux variability is consistent with zero";
    $suminfo{comm}{"VAR_PROB_1"   } = "Probability that the EPIC band 1 flux variability is consistent with zero";
    $suminfo{comm}{"VAR_PROB_2"   } = "Probability that the EPIC band 2 flux variability is consistent with zero";
    $suminfo{comm}{"VAR_PROB_3"   } = "Probability that the EPIC band 3 flux variability is consistent with zero";
    $suminfo{comm}{"VAR_PROB_4"   } = "Probability that the EPIC band 4 flux variability is consistent with zero";
    $suminfo{comm}{"VAR_PROB_5"   } = "Probability that the EPIC band 5 flux variability is consistent with zero";
    $suminfo{comm}{"FRATIO"       } = "EPIC flux ratio";
    $suminfo{comm}{"FRATIO_1"     } = "EPIC band 1 flux ratio";
    $suminfo{comm}{"FRATIO_2"     } = "EPIC band 2 flux ratio";
    $suminfo{comm}{"FRATIO_3"     } = "EPIC band 3 flux ratio";
    $suminfo{comm}{"FRATIO_4"     } = "EPIC band 4 flux ratio";
    $suminfo{comm}{"FRATIO_5"     } = "EPIC band 5 flux ratio";
    $suminfo{comm}{"FRATIO_ERR"   } = "1-sigma error on FRATIO";
    $suminfo{comm}{"FRATIO_1_ERR" } = "1-sigma error on FRATIO_1";
    $suminfo{comm}{"FRATIO_2_ERR" } = "1-sigma error on FRATIO_2";
    $suminfo{comm}{"FRATIO_3_ERR" } = "1-sigma error on FRATIO_3";
    $suminfo{comm}{"FRATIO_4_ERR" } = "1-sigma error on FRATIO_4";
    $suminfo{comm}{"FRATIO_5_ERR" } = "1-sigma error on FRATIO_5";
    $suminfo{comm}{"FLUXVAR"      } = "Largest EPIC flux difference in terms of sigma";
    $suminfo{comm}{"FLUXVAR_1"    } = "Largest EPIC band 1 flux difference in terms of sigma";
    $suminfo{comm}{"FLUXVAR_2"    } = "Largest EPIC band 2 flux difference in terms of sigma";
    $suminfo{comm}{"FLUXVAR_3"    } = "Largest EPIC band 3 flux difference in terms of sigma";
    $suminfo{comm}{"FLUXVAR_4"    } = "Largest EPIC band 4 flux difference in terms of sigma";
    $suminfo{comm}{"FLUXVAR_5"    } = "Largest EPIC band 5 flux difference in terms of sigma";
    $suminfo{comm}{"MJD_FIRST"    } = "Modified Julian Date JD-2400000.5 of the observation start";
    $suminfo{comm}{"MJD_LAST"     } = "Modified Julian Date JD-2400000.5 of the observation end";
    $suminfo{comm}{"REVOLUT"      } = "XMM-Newton revolution number of the observation";
    $suminfo{comm}{"PA_PNT"       } = "Mean position angle of the spacecraft";
    $suminfo{comm}{"PN_SUBMODE"   } = "EPIC/pn submode used for the observation";
    $suminfo{comm}{"M1_SUBMODE"   } = "EPIC/MOS1 submode used for the observation";
    $suminfo{comm}{"M2_SUBMODE"   } = "EPIC/MOS2 submode used for the observation";
    $suminfo{comm}{"PN_FILTER"    } = "EPIC/pn filter used for the observation";
    $suminfo{comm}{"M1_FILTER"    } = "EPIC/MOS1 filter used for the observation";
    $suminfo{comm}{"M2_FILTER"    } = "EPIC/MOS2 filter used for the observation";

    # explicitly connect those column names to emldetect columns which are
    # addressed in loops later-on (can be changed here easily ...)
    $suminfo{emlname}{$_} = $_
	for ("ML_ID_SRC"    ,
	     "BOX_ID_SRC"   ,
	     "RA"           ,
	     "DEC"          ,
	     "RADEC_ERR"    ,
	     "LII"          ,
	     "BII"          ,
	     "X_IMA"        ,
	     "X_IMA_ERR"    ,
	     "Y_IMA"        ,
	     "Y_IMA_ERR"    ,
	     "DIST_NN"      ,
	     "N_BLEND"      ,
	     "FLUX"         ,
	     "FLUX_ERR"     ,
	     "RATE"         ,
	     "RATE_ERR"     ,
	     "DET_ML"       ,
	     "HR1"          ,
	     "HR1_ERR"      ,
	     "HR2"          ,
	     "HR2_ERR"      ,
	     "HR3"          ,
	     "HR3_ERR"      ,
	     "HR4"          ,
	     "HR4_ERR"      ,
	     "ONTIME"       ,
	     "PILEUP"       ,
	     "MASKFRAC"     ,
	     "OFFAX"        ,
	     "MJD_FIRST"    ,
	     "MJD_LAST"     ,
	     "FLAG"         );
    $suminfo{emlname}{"CTS"          } = "SCTS"         ;
    $suminfo{emlname}{"CTS_ERR"      } = "SCTS_ERR"     ;
    $suminfo{emlname}{"EXTENT"       } = "EXT"          ;
    $suminfo{emlname}{"EXTENT_ERR"   } = "EXT_ERR"      ;
    $suminfo{emlname}{"EXTENT_ML"    } = "EXT_ML"       ;
    $suminfo{emlname}{"EXP"          } = "EXP_MAP"      ;
    $suminfo{emlname}{"BG"           } = "BG_MAP"       ;
    $suminfo{emlname}{"VIG"          } = "VIGNETTING"   ;

    # and the other way round, for column format only
    $suminfo{namesum}{$_} = $_
	for ("ML_ID_SRC"    ,
	     "BOX_ID_SRC"   ,
	     "RA"           ,
	     "DEC"          ,
	     "RADEC_ERR"    ,
	     "LII"          ,
	     "BII"          ,
	     "X_IMA"        ,
	     "X_IMA_ERR"    ,
	     "Y_IMA"        ,
	     "Y_IMA_ERR"    ,
	     "DIST_NN"      ,
	     "N_BLEND"      ,
	     "MJD_FIRST"    ,
	     "MJD_LAST"     ,
	     "FLAG"         ,
	     "RAWX"         ,
	     "RAWY"         ,
	     "CCDNR"        ,
	     "EEF"          );
    $suminfo{namesum}{"FLUX"         } = "EP_FLUX"         ;
    $suminfo{namesum}{"FLUX_ERR"     } = "EP_FLUX_ERR"     ;
    $suminfo{namesum}{"RATE"         } = "EP_RATE"         ;
    $suminfo{namesum}{"RATE_ERR"     } = "EP_RATE_ERR"     ;
    $suminfo{namesum}{"SCTS"         } = "EP_CTS"          ;
    $suminfo{namesum}{"SCTS_ERR"     } = "EP_CTS_ERR"      ;
    $suminfo{namesum}{"DET_ML"       } = "EP_DET_ML"       ;
    $suminfo{namesum}{"EXT"          } = "EXTENT"          ;
    $suminfo{namesum}{"EXT_ERR"      } = "EXTENT_ERR"      ;
    $suminfo{namesum}{"EXT_ML"       } = "EXTENT_ML"       ;
    $suminfo{namesum}{"HR1"          } = "EP_HR1"          ;
    $suminfo{namesum}{"HR1_ERR"      } = "EP_HR1_ERR"      ;
    $suminfo{namesum}{"HR2"          } = "EP_HR2"          ;
    $suminfo{namesum}{"HR2_ERR"      } = "EP_HR2_ERR"      ;
    $suminfo{namesum}{"HR3"          } = "EP_HR3"          ;
    $suminfo{namesum}{"HR3_ERR"      } = "EP_HR3_ERR"      ;
    $suminfo{namesum}{"HR4"          } = "EP_HR4"          ;
    $suminfo{namesum}{"HR4_ERR"      } = "EP_HR4_ERR"      ;
    $suminfo{namesum}{"EXP_MAP"      } = "PN_EXP"          ;
    $suminfo{namesum}{"BG_MAP"       } = "PN_BG"           ;
    $suminfo{namesum}{"ONTIME"       } = "PN_ONTIME"       ;
    $suminfo{namesum}{"PILEUP"       } = "PN_PILEUP"       ;
    $suminfo{namesum}{"MASKFRAC"     } = "PN_MASKFRAC"     ;
    $suminfo{namesum}{"OFFAX"        } = "PN_OFFAX"        ;
    $suminfo{namesum}{"VIGNETTING"   } = "PN_1_VIG"        ;


    ############################################################################
    # SSC ONLY: flag sources and copy flag column to emldetect source list
    my $dummy = qx/dpssflag -h 2>&1/;                           ### experimental
    if ( $? eq 0) {
        SAS::message($SAS::AppMsg, $SAS::VerboseMsg,
                     "- Flagging sources via DPSSFLAG.");
        # create temporary ID_INST column of integer type
        $emlfits = 
            SimpleFITS->open($inlist, access=>"readwrite", type=>"table");
        if ($emlfits->status() != 0) {
            SAS::warning("FileIOError", "Cannot work on $inlist. Please make".
                         " sure that it exists and you have read and write".
                         " access. Skipping dpssflag.");
        } else {
            $emlfits
                ->readcol("ID_INST", {}, my $idinstorig);
            my @tmpinsts = map { substr $_, -1 } @$idinstorig;
            my $colnum = $emlfits->colnum("ID_INST");
            $emlfits
                ->insertcol({TTYPE=>"ID_INSTorig", TFORM=>"20A"})
                ->writecol ("ID_INSTorig", {}, $idinstorig)      ## just in case
                ->delcol   ("ID_INST")
                ->insertcol({TTYPE=>"ID_INST",     TFORM=>"J"}, $colnum)
                ->writecol ("ID_INST",     {}, \@tmpinsts)
                ->close()
                ;
            # (old) long input list may contain HIERARCH coordinate keywords
            # in their first extension. dal & cfitsio don't handle them, so
            # dpssflag would exit with a 202 error message if finding any.
            # Delete them one by one (SimpleFITS stops on the first
            # non-existing keyword, if given as a command list).
            $emlfits = SimpleFITS->open($inlist, access=>"readwrite");
            $emlfits->delkey('OARA??**');
            $emlfits->delkey('OARA??**');
            $emlfits->delkey('OARA??**');
            $emlfits->close();     # "empirically"
            $emlfits = SimpleFITS->open($inlist, access=>"readwrite");
            $emlfits->delkey('OADE??**');
            $emlfits->delkey('OADE??**');
            $emlfits->delkey('OADE??**');
            $emlfits->close();
            # Now, we can start flagging.
            my $dpssflag =
                "dpssflag".
                " srclisttable='${inlist}:SRCLIST'".
                " extlimit=$flag_extlimit".
                " mfraclimit=$flag_mfraclimit".
                " brcolumn='RATE'".
                " brfactor=$flag_brfactor".
                " brlimit=$flag_brlimit".
                " op1='sqrt'".
                " op2='mul'".
                " minrad=$flag_minrad".
                " maxrad=$flag_maxrad".
                " extfactor=$flag_extfactor".
                " maxextrad=$flag_maxextrad".
                " fixext=$flag_fixext".
                " brexfraction=$flag_brexfraction".
                " detmlfraction=$flag_detmlfraction".
                " badcolumn='CUTRAD'".
                " badfactor=$flag_badfactor".
                " withflag1=1".
                " withflag2=1".
                " withflag3=1".
                " withflag4=1".
                " withflag5=1".
                " withflag6=1".
                " withflag7=1".
                " withflag8=1".
                " withflag9=1".
                " withflag10=0".
                " withflag11=0".
                " withflag12=0";
            SAS::message($SAS::AppMsg, $SAS::VerboseMsg, "Calling: $dpssflag");
            system("$dpssflag");
            if ($? != 0) {
                SAS::warning("TaskError",  "Task ended in error: $dpssflag");
            }
            $emlfits = 
                SimpleFITS->open($inlist, access=>"readwrite", type=>"table");
            $emlfits
                ->delcol   ("ID_INST")
                ->delcol   ("ID_INSTorig")
                ->insertcol({TTYPE=>"ID_INST",     TFORM=>"D"}, $colnum)
                ->writecol ("ID_INST",     {}, $idinstorig)
                ->writekey("TDISP".$colnum, "F11.0",
                           "Display ID_INST like integers")
                ->close();
        }                                               # end if file accessable
    }                                                          # end if dpssflag
    ############################################################################


    ########################################
    # open files and create file structures

    # open first extension of the emldetect list (read-only)
    $infits = Astro::FITS::CFITSIO::open_file($inlist, READONLY, $status);
    if ($status) {
        die SAS::error("FileIOerror", "Cannot read $inlist. Please make sure".
                       " that it exists and you have read access.");
    }

    # create file structure of emldetect-type source list
    Astro::FITS::CFITSIO::fits_create_file($emlfits, "!".$outlist, $status);
    if ($status) {
        die SAS::error("FileIOerror", "Cannot work on $outlist. Please make".
                       " sure that you have write access.");
    }
    # copy primary header to summary source list
    $infits->copy_header($emlfits, $status);

    # create file structure of summary source list
    Astro::FITS::CFITSIO::fits_create_file($sumfits, "!".$outlistsum, $status);
    if ($status) {
        die SAS::error("FileIOerror", "Cannot work on $outlistsum.");
    }
    # copy primary header to summary source list
    $infits->copy_header($sumfits, $status);
    $sumfits->close_file($status);
    # Switch to the more flexible SimpleFITS to create the srclist table
    # extension and make sure that the keywords are ordered
    $sumfits = SimpleFITS->open($outlistsum, access=>"readwrite");
    ($sumfits->status() == 0) 
        or SAS::error("FileIOerror", "Cannot work on $outlistsum.");
    $sumfits->createtab("SRCLIST");
    for (0..$#sumnames) {
        $sumfits
            ->insertcol({ TTYPE=>$sumnames[$_], 
                          TFORM=>$suminfo{form}{$sumnames[$_]},
                          TUNIT=>$suminfo{unit}{$sumnames[$_]} })
            ->writekey("TCOMM".($_+1), $suminfo{comm}{$sumnames[$_]})
            ;
    }


    # get HR??DEF keywords
    # If not defined: parameter value or default
    # Warning about readkey: If a keyword is undefined, SimpleFits stops
    # reading the header.
    $fullheader = $infits->read_header();
    $infits->movrel_hdu(1, my $tmpvar, $status);
    $listheader = $infits->read_header();
    if (defined($$fullheader{"HRPNDEF"})) {
        $hhash{"HRPNDEF"} = $$fullheader{"HRPNDEF"};
    } else {
        if (defined($addpar[0])) {
            $hhash{"HRPNDEF"} = $addpar[0];
        } else {
            $hhash{"HRPNDEF"} = "1 2 2 3 3 4 4 5";
        }
        SAS::warning("KeywordMissing", 
                     "Keyword HRPNDEF not found - using default value.");
    }
    $hhash{"HRPNDEF"} =~ tr/0-9 //cd;
    $hhash{"HRPNDEF"} =~ s/^\s+//;
    if (defined($$fullheader{"HRM1DEF"})) {
        $hhash{"HRM1DEF"} = $$fullheader{"HRM1DEF"};
    } else {
        if (defined($addpar[1])) {
            $hhash{"HRM1DEF"} = $addpar[1];
        } else {
            $hhash{"HRM1DEF"} = "1 2 2 3 3 4 4 5";
        }
        SAS::warning("KeywordMissing", 
                     "Keyword HRM1DEF not found - using default value.");
    }
    $hhash{"HRM1DEF"} =~ tr/0-9 //cd;
    $hhash{"HRM1DEF"} =~ s/^\s+//;
    if (defined($$fullheader{"HRM2DEF"})) {
        $hhash{"HRM2DEF"} = $$fullheader{"HRM2DEF"};
    } else {
        if (defined($addpar[2])) {
            $hhash{"HRM2DEF"} = $addpar[2];
        } else {
            $hhash{"HRM2DEF"} = "1 2 2 3 3 4 4 5";
        }
        SAS::warning("KeywordMissing", 
                     "Keyword HRM2DEF not found - using default value.");
    }
    $hhash{"HRM2DEF"} =~ tr/0-9 //cd;
    $hhash{"HRM2DEF"} =~ s/^\s+//;
    # hrdef structure (cf. emldetect/eml_io.f90:
    @{$hrdef{1}} = split(/ /,$hhash{"HRPNDEF"});
    @{$hrdef{2}} = split(/ /,$hhash{"HRM1DEF"});
    @{$hrdef{3}} = split(/ /,$hhash{"HRM2DEF"});
    # can close the (read-only) first extension of the main file now
    $infits->close_file($status);


    # get reference coordinates: from image header via hhash, if possible, or
    # alternatively from source-list header
    if (defined($$hhash{"REFXRA"})) {
        $ra_ref = $$hash{"REFXRA"};
    } else {
        if (defined($$fullheader{"TCRVAL1"})) {
            $ra_ref = $$fullheader{"TCRVAL1"};
        } else {
            $ra_ref = $mynull;
	    SAS::warning("KeywordMissing", 
			 "Reference RA not found. DIST_REF will be undefined.");
        }
    }
    if (defined($$hhash{"REFYDE"})) {
        $de_ref = $$hash{"REFYDE"};
    } else {
        if (defined($$fullheader{"TCRVAL2"})) {
            $de_ref = $$fullheader{"TCRVAL2"};
        } else {
            $de_ref = $mynull;
	    SAS::warning("KeywordMissing", 
			 "Reference DEC not found. DIST_REF will be undefined.");
        }
    }


    # read input source list (read-only); more fun with SimpleFITS
    $infits = SimpleFITS->open($inlist, access=>"readonly");
    $infits
        ->move("SRCLIST")
        ->ncols(my $ncol);
    # insert columns into hash of arrays and close the handler
    for my $icol (1..$ncol) {
        $srcnames[$icol-1] = ($$listheader{"TTYPE".$icol} =~ s/\s|'//gr);
        @{$srclist{$srcnames[$icol-1]}} = $infits->readcol($icol);
    }
    $infits->close;
    # add table extension to output file
    $emlfits = SimpleFITS->open($outlist, access=>"readwrite");
    ($emlfits->status() == 0) 
        or SAS::error("FileNotFound", "Could not open $outlist. ".
                      "File not found or unreadable.");
    $emlfits->createtab("SRCLIST");
        

    # determine number of energy bands:
    $nbands = List::Util::max @{$srclist{"ID_BAND"}};

    ## INSERT SUMMARY ROWS PER POINTING, initialized with values of first row
    # loop over unique source IDs
    # Both loops (over sources and over pointings) are run in reverse order,
    # so we won't mess up the row indices by inserting the new summary rows.
    %seenids = ();
    @srcinids = reverse(grep { ! $seenids{ $_ }++ } @{$srclist{"ML_ID_SRC"}});

    for my $srcid (@srcinids) {                    ### reverse loop over sources

        # indices of the rows which have ML_ID_SRC $srcid
        my ( @srcinds ) = 
            grep { ${$srclist{"ML_ID_SRC"}}[$_] eq $srcid }
            0..$#{$srclist{"ML_ID_SRC"}};
        my @instslice = @{$srclist{"ID_INST"}}[@srcinds];

        # Initialize maximum of pointing DET_MLs with the all-EPIC DET_ML.
        # Total likelihood or at least one pointing likelihood have to be larger
        # than $mindetml to include a detection in the final source list.
        $tmpmaxdetml[$srcid] = ${$srclist{"DET_ML"}}[$srcinds[0]];

	## Insert N_BLEND column: number of simultaneously fit sources
	$tmpnblend[$srcid] = 
	    grep { ${$srclist{"ID_CLUSTER"}}[$_]
		       eq ${$srclist{"ID_CLUSTER"}}[$srcinds[0]] 
		       and ${$srclist{"ID_INST"}}[$_] eq 0 
		       and ${$srclist{"ID_BAND"}}[$_] eq 0 }
	    0..$#{$srclist{"ML_ID_SRC"}};

        ## DEFINE ROWS PER POINTING
        # loop over unique pointings per source ID
        # - modify summary rows last
        my %seenpts = ();
        # get pointing IDs:
        # - use an ML_ID_SRC slice of ID_INST
        # - ignore last digit of ID_INST
        @pointings = 
            reverse(grep { ! $seenpts{ (substr $_,0,(length($_)-1)) }++ }  
                    @{$srclist{"ID_INST"}}[@srcinds]);
        foreach my $instidpnt (@pointings) {     ### reverse loop over pointings

            $instidpnt = (substr $instidpnt,0,(length($instidpnt)-1));

            if ($instidpnt ne "") {                 ### begin: rows per pointing

            # row numbers of this pointing (indices starting with 0)
            my @pntinds = 
                @srcinds[(grep
                          { (substr $instslice[$_], 0,
                             (length($instslice[$_])-1))
                                eq "$instidpnt" } 
                          0..$#instslice)];

            # calculate detection likelihood of the pointing and compare it to
            # previous values
            $tmpdetml
                = likelihood( # dofs/2, sum(likelihoods)
                              (2+(${$srclist{"EXT"}}[$pntinds[0]]>0)
                               +(scalar @pntinds))/2.0,
                              List::Util::sum @{$srclist{"DET_ML"}}[@pntinds] );
            # maximum likelihood of pointings
            $tmpmaxdetml[$srcid] = $tmpdetml
                if ($tmpdetml > $tmpmaxdetml[$srcid]);

            ## insert summary rows per pointing: one total, one per instrument

                # new rows, initialized with values from the next row
                for my $col (@srcnames) {
                    splice @{$srclist{$col}}, $pntinds[0], 0,
                    (${$srclist{$col}}[$pntinds[0]]) x $nnewrows;
                }

                # ID_INST: abbreviated OBS_ID + INST_ID
                @{$srclist{"ID_INST"}}[map { $pntinds[0]+$_ } (0..$nnewrows-1)] 
                    = map { "$instidpnt".$_ } (0..$nnewrows-1);

                # ID_BAND: zero in summary rows
                @{$srclist{"ID_BAND"}}[map { $pntinds[0]+$_ } (0..$nnewrows-1)]
                    = (0) x $nnewrows;

            # store detection likelihood
            ${$srclist{"DET_ML"}}[$pntinds[0]] = $tmpdetml;

            }
        }
    }



    ## UPDATE SOURCE LIST: MODIFY COLUMN BY COLUMN
    # relevant columns: IDs, those to be re-defined, those affected by
    # MASKFRAC, and those used to derive other values.
    # others: copied to summary file

## Long integers are not supported by the DAL, which also means: The source
## list cannot be read by Fortran tasks, if ID_INST is converted into long
## integer. Stick to the workaround: floating point with double precision and
## display format TDISP F11.0.

    # number of pointings involved in the stack: number of unique pointing ids
    # (alternatively:
    #  just pass the information from edetect_stack.pl to stack_sourcelist.pl)
    %seenids = ();
    $sum_nobs =
        (grep { ! $seenids{ (substr $_,0,(length($_)-1)) }++ }  
         @{$srclist{"ID_INST"}}) - 1;

    # main loop over unique source IDs
    for my $srcid (@srcinids) {                    ### reverse loop over sources

	$sum_nexpos = 0;

        # indices of the rows which have ML_ID_SRC $srcid
        my ( @srcinds ) = grep { ${$srclist{"ML_ID_SRC"}}[$_] eq $srcid }
                          0..$#{$srclist{"ML_ID_SRC"}};

        # only sources with detection likelihood above $mindetml
        if ($tmpmaxdetml[$srcid] >= $mindetml) {

        my @instslice = @{$srclist{"ID_INST"}}[@srcinds];

        # initialize (empty) arrays of pointing summary values, which will be
        # used for the total summary later-on
        @{$instrows{1}} = ();
        @{$instrows{2}} = ();
        @{$instrows{3}} = ();

        ## DEFINE ROWS PER POINTING
        # loop over unique pointings per source ID
        # - modify summary rows last
        my %seenpts = ();
        # get pointing IDs:
        # - use an ML_ID_SRC slice of ID_INST
        # - ignore last digit of ID_INST
        @pointings = 
            grep { ! $seenpts{ (substr $_,0,(length($_)-1)) }++ }  
            @{$srclist{"ID_INST"}}[@srcinds];

        # new source identifier in catalogue format:
        # "3" + OBS_ID (10 digits) + ML_ID_SRC (5 digits, one more than in 3XMM)
        my $sum_srcid = "3"
            .sprintf("%010i", substr List::Util::min
                     (grep { $_>99 } @pointings), 0, -1)
            .sprintf("%05i", $srcid);

        # number of valid pointings per detection:
        # number of elements in @pointings minus the all-EPIC entry
        $sum_ncontrib = $#pointings;

	# distance to the reference coordinates
	$dist_ref =
	    sin(${$srclist{"DEC"}}[$srcinds[0]]*PI180)*sin($de_ref*PI180)
	    + cos(${$srclist{"DEC"}}[$srcinds[0]]*PI180)*cos($de_ref*PI180)
	    * cos((${$srclist{"RA"}}[$srcinds[0]] - $ra_ref)*PI180);
	$dist_ref = 
	    atan2( sqrt(1 - $dist_ref*$dist_ref), $dist_ref )    ## perl acos
	    / PI180 * 60.0;
	
        # intialize ontime and pile-up values per instrument
        @tmpontime = (0,0,0);
        @tmppileup = (-$mynull,-$mynull,-$mynull);

        # initialize merged 8 & 9 flags (pointing and band summaries)
        @tmpflagpnt  = ("FF") x $#insts;
        for my $ii (1..$#insts) {
            for my $ib (1..$nbands) {
                $tmpflagband[$ii.$ib] = "FF";
            }
        }
        ${$srclist{"STACK_FLAG"}}[$srcinds[0]] = 0;   # used internally only

        foreach my $instidpnt (reverse(@pointings)) {   ###  loop over pointings

	    my $sum_ninst = 0;
            $instidpnt = (substr $instidpnt,0,(length($instidpnt)-1));
            if ($instidpnt ne "") {                 ### begin: rows per pointing

                # row numbers of this pointing (indices starting with 0),
                # now including the new summary rows:
                # $pntinds[0] is the all-EPIC summary of the pointing,
                # $pntinds[1..3] the pn, MOS1, MOS2 summary rows of the pointing
                my @pntinds = 
                    @srcinds[(grep { (substr $instslice[$_], 0,
                                  (length($instslice[$_])-1)) eq "$instidpnt" } 
                              0..$#instslice)];

                # add row indices of summary rows to array of valid indices
                unshift @srcvalinds, @pntinds[0..$nnewrows-1];

                # initialize merged flag (instruments per pointing)
                $tmpflaginst = "FFFFFFFFF";

#               ################################################################
                ## 1. set final column values PER POINTING PER INSTRUMENT
                #                                                    (="pntins")
                for my $nr (1..$nnewrows-1) {     ### loop over new summary rows

                    $thisrow = $pntinds[0]+$nr;

                    # rows corresponding to this instrument
                    my @instinds =
                        @srcinds[(grep
                                  { $instslice[$_] eq "$instidpnt"."$nr"
                                    && ${$srclist{"ID_BAND"}}[$srcinds[$_]] != 0
                                  } 
                                  0..$#instslice)];
                    if (@instinds) {               ### begin: instrument defined

                        # PREPARATORY WORK

                        # add the instrument row numbers to the index arrays
                        push @{$instrows{$nr}}, @instinds;

			# add the number unique instrument ids to sum_nexpos
			%seenpts = ();
			$sum_ninst += (grep { ! $seenpts{ $_ }++ }
				       @{$srclist{"ID_INST"}}[@instinds]);

                        # initialize parameter columns with values from first
                        # instrument row
                        ${$srclist{$srcnames[$_-1]}}[$thisrow] = 
                            ${$srclist{$srcnames[$_-1]}}[$instinds[0]]
                            for ($startcol..$ncol);

                        # temporary values of EXP_MAP, MASKFRAC, used for checks
                        $tmpexpmap = 
                            List::Util::sum @{$srclist{"EXP_MAP"}}[@instinds];
                        $tmpmaskfrac = 
                            List::Util::min @{$srclist{"MASKFRAC"}}[@instinds];

                        # up to four hardness ratios
                        for my $ie (0..$nbands-2) {
                                    # loop over energy bands (max. $nband-1 HRs)
                            $tmphrval[$ie] = $mynull;
                            $tmphrerr[$ie] = $mynull;
                            # calculate hardness ratio, if @hrdef is set 
                            if (defined(${$hrdef{$nr}}[2*$ie])
                                && defined(${$hrdef{$nr}}[2*$ie+1])
                                && "0"."${$hrdef{$nr}}[2*$ie]"   > 0
                                && "0"."${$hrdef{$nr}}[2*$ie+1]" > 0
                                && "0"."${$hrdef{$nr}}[2*$ie]"   < 1e99
                                && "0"."${$hrdef{$nr}}[2*$ie+1]" < 1e99) {
                                # ... and if corresponding rows are found
                                if( ${$srclist{"ID_BAND"}}
                                    [$instinds[0]-1+${$hrdef{$nr}}[2*$ie]]
                                    == ${$hrdef{$nr}}[2*$ie]
                                    && ${$srclist{"ID_BAND"}}
                                    [$instinds[0]-1+${$hrdef{$nr}}[2*$ie+1]]
                                    == ${$hrdef{$nr}}[2*$ie+1] )
                                {
                                # indices: rows are sorted by band number
                                # -> first row of the instrument block
                                #    plus band indices minus one (start with 0)
                                my $lowind = 
                                    $instinds[0]-1 + ${$hrdef{$nr}}[2*$ie];
                                my $higind = 
                                    $instinds[0]-1 + ${$hrdef{$nr}}[2*$ie+1];
                                # max(0.,rate):
                                my $ratelowval = 
                                    (${$srclist{"RATE"}}[$lowind]>=0) 
                                    ? ${$srclist{"RATE"}}[$lowind] : 0;
                                my $ratehigval = 
                                    (${$srclist{"RATE"}}[$higind]>=0) 
                                    ? ${$srclist{"RATE"}}[$higind] : 0;
                                # HR
                                my $tmpsum = $ratelowval+$ratehigval;
                                if ($tmpsum != 0) {
                                    $tmphrval[$ie] =
                                        ($ratehigval-$ratelowval) / $tmpsum ;
                                # HR error
                                    $tmphrerr[$ie] = 
                                        2.0*sqrt(${$srclist{"RATE"}}[$lowind]
                                                *${$srclist{"RATE"}}[$lowind]
                                               *${$srclist{"RATE_ERR"}}[$lowind]
                                               *${$srclist{"RATE_ERR"}}[$lowind]
                                                 +
                                                 ${$srclist{"RATE"}}[$higind]
                                                *${$srclist{"RATE"}}[$higind]
                                               *${$srclist{"RATE_ERR"}}[$higind]
                                               *${$srclist{"RATE_ERR"}}[$higind]
                                        ) / $tmpsum/$tmpsum ;
                                }               
                            }}                                  # end if defined
                        }                           # end loop over energy bands

                        # add ontime to array of pointings sum
                        $tmpontime[$nr-1] += ${$srclist{"ONTIME"}}[$thisrow]
                            if (${$srclist{"ONTIME"}}[$thisrow] > 0.0);

                        # COLUMNS ##############################################
                # SCTS: sum; null if exp_map<=0 or maskfrac<threshold   (pntins)
                        ${$srclist{"SCTS"}}[$thisrow] =
                            (defined($tmpmaskfrac) && $tmpmaskfrac>=$maskthr &&
                             defined($tmpexpmap)   && $tmpexpmap>0)
                            ? (List::Util::sum @{$srclist{"SCTS"}}[@instinds])
                            : $mynull;
                # SCTS_ERR: square root of quadratic sum or null
                        ${$srclist{"SCTS_ERR"}}[$thisrow] =
                            (defined($tmpmaskfrac) && $tmpmaskfrac>=$maskthr &&
                             defined($tmpexpmap) && $tmpexpmap>0)
                            ? (sqrt(List::Util::sum 
                                    ( map {(defined($_)) ? $_*$_ : 0.0}
                                      (@{$srclist{"SCTS_ERR"}}[@instinds]) ) ))
                            : $mynull;
                # DET_ML: equivalent likelihood via gamma function or NULL
                        ${$srclist{"DET_ML"}}[$thisrow] =
                            (defined($tmpmaskfrac) && $tmpmaskfrac>=$maskthr &&
                             defined($tmpexpmap) && $tmpexpmap>0)
                            ? likelihood(   # dofs/2, sum(likelihoods)
                                (2+(${$srclist{"EXT"}}[$instinds[0]]>0)
                                +(scalar @instinds))/2.0,
                                List::Util::sum @{$srclist{"DET_ML"}}[@instinds]
                               ) 
                            : $mynull;
                # BG_MAP: sum; null if exp_map<=0                       (pntins)
                        ${$srclist{"BG_MAP"}}[$thisrow] =
                            (defined($tmpexpmap) && $tmpexpmap>0)
                            ? (List::Util::sum @{$srclist{"BG_MAP"}}[@instinds])
                            : $mynull;
                # EXP_MAP: sum; null if negative                        (pntins)
                        ${$srclist{"EXP_MAP"}}[$thisrow] =
                            (defined($tmpexpmap) && $tmpexpmap>0)
                            ? $tmpexpmap
                            : $mynull;
                # FLUX: sum; null if exp_map<=0 or maskfrac<threshold [later-on]
                        ${$srclist{"FLUX"}}[$thisrow] = 
                            List::Util::sum
                                 ( map {(defined ${$srclist{"FLUX"}}[$_]
                                            && ${$srclist{"FLUX"}}[$_] < 1e99
                                            && ${$srclist{"EXP_MAP"}}[$_] > 0)
                                            ? ${$srclist{"FLUX"}}[$_]
                                   : 0.0}
                                   @instinds );
                # FLUX_ERR: square root of quadratic sum or null        (pntins)
                        ${$srclist{"FLUX_ERR"}}[$thisrow] =
                            sqrt(List::Util::sum 
                                 ( map {(defined ${$srclist{"FLUX_ERR"}}[$_]
                                          && ${$srclist{"FLUX_ERR"}}[$_] < 1e99
                                            && ${$srclist{"EXP_MAP"}}[$_] > 0)
                                            ? ${$srclist{"FLUX_ERR"}}[$_]
                                            *${$srclist{"FLUX_ERR"}}[$_]
                                   : 0.0}
                                   @instinds ));
                # RATE: sum; null if exp_map<=0 or maskfrac<threshold   (pntins)
                        ${$srclist{"RATE"}}[$thisrow] =
                            (defined($tmpmaskfrac) && $tmpmaskfrac>=$maskthr &&
                             defined($tmpexpmap) && $tmpexpmap>0)
                            ? (List::Util::sum @{$srclist{"RATE"}}[@instinds])
                            : $mynull;
                # RATE_ERR: square root of quadratic sum or null        (pntins)
                        ${$srclist{"RATE_ERR"}}[$thisrow] =
                            (defined($tmpmaskfrac) && $tmpmaskfrac>=$maskthr &&
                             defined($tmpexpmap) && $tmpexpmap>0)
                            ? (sqrt(List::Util::sum 
                                    ( map {(defined($_)) ? $_*$_ : 0.0}
                                      (@{$srclist{"RATE_ERR"}}[@instinds]) ) ))
                            : $mynull;
                # HR1: calculated above, set to null if lower than -10
                        ${$srclist{"HR1"}}[$thisrow] =
                            ($tmphrval[0] >= -10.0)
                            ? $tmphrval[0]
                            : $mynull;
                # HR1_ERR                                               (pntins)
                        ${$srclist{"HR1_ERR"}}[$thisrow] =
                            ($tmphrerr[0] >= -10.0)
                            ? $tmphrerr[0]
                            : $mynull;
                # HR2                                                   (pntins)
                        ${$srclist{"HR2"}}[$thisrow] =
                            ($tmphrval[1] >= -10.0)
                            ? $tmphrval[1]
                            : $mynull;
                # HR2_ERR                                               (pntins)
                        ${$srclist{"HR2_ERR"}}[$thisrow] =
                            ($tmphrerr[1] >= -10.0)
                            ? $tmphrerr[1]
                            : $mynull;
                # HR3                                                   (pntins)
                        ${$srclist{"HR3"}}[$thisrow] =
                            ($tmphrval[2] >= -10.0)
                            ? $tmphrval[2]
                            : $mynull;
                # HR3_ERR                                               (pntins)
                        ${$srclist{"HR3_ERR"}}[$thisrow] =
                            ($tmphrerr[2] >= -10.0)
                            ? $tmphrerr[2]
                            : $mynull;
                # HR4                                                   (pntins)
                        ${$srclist{"HR4"}}[$thisrow] =
                            ($tmphrval[3] >= -10.0)
                            ? $tmphrval[3]
                            : $mynull;
                # HR4_ERR                                               (pntins)
                        ${$srclist{"HR4_ERR"}}[$thisrow] =
                            ($tmphrerr[3] >= -10.0)
                            ? $tmphrerr[3]
                            : $mynull;
                # RAWX, RAWY, CCDNR: unchanged                          (pntins)
                        #
		# OFFAX: divided by 60.                                 (pntins)
                        ${$srclist{"OFFAX"}}[$thisrow] =
                            (defined(${$srclist{"OFFAX"}}[$thisrow]))
                            ? (${$srclist{"OFFAX"}}[$thisrow]/60.)
                            : $mynull;
                # MASKFRAC per pointing: minimum                        (pntins)
                        ${$srclist{"MASKFRAC"}}[$thisrow] = $tmpmaskfrac;
                # EEF & VIGNETTING: null for instrument summaries       (pntins)
                        ${$srclist{"EEF"       }}[$thisrow] = $mynull;
                        ${$srclist{"VIGNETTING"}}[$thisrow] = $mynull;
                # ONTIME: set to zero if negative                       (pntins)
                        ${$srclist{"ONTIME"    }}[$thisrow] = 0.0 
                            if (${$srclist{"ONTIME"}}[$thisrow] < 0.0);
		# PILEUP: use the factor from one of the energy-level rows 
		#         for point-like sources and the sum of the
		#         image-level pile-up estimates for extended sources
			${$srclist{"PILEUP"    }}[$thisrow] = 
			    (${$srclist{"SCTS"}}[$thisrow] < 1e99
			     && defined ${$srclist{"PILEUP"}}[$instinds[0]])
			    ? (${$srclist{"EXT"}}[$instinds[0]]>0) 
			      ? (List::Util::sum 
				               @{$srclist{"PILEUP"}}[@instinds])
			      : ${$srclist{"RATE"}}[$thisrow] *
			        ${$srclist{"PILEUP"}}[$instinds[0]]
                            : $mynull  ;
                # FLAG: merged band flags, if applicable (flags 1-9 only)
                        if (${$srclist{"FLAG"}}[$thisrow] =~ /F|T/) {
                            ${$srclist{"FLAG"}}[$thisrow] =
                                substr ${$srclist{"FLAG"}}[$thisrow], 0, 9;
                            for $itmp (@instinds) {
                                # current flag
                                ${$srclist{"FLAG"}}[$thisrow] =
                                    merge_flags(${$srclist{"FLAG"}}[$thisrow],
                                                ${$srclist{"FLAG"}}[$itmp]   );
                                # merge flags 8 & 9 with band summary
                                $tmpflagband[$nr.${$srclist{"ID_BAND"}}[$itmp]]
                                    = merge_flags(
                                    $tmpflagband[$nr.${$srclist{"ID_BAND"}}
                                                 [$itmp]],
                                    substr(${$srclist{"FLAG"}}[$itmp],7,2));
                            }
                            # merge row flag with merged camera flag
                            $tmpflaginst = 
                                merge_flags($tmpflaginst,
                                            ${$srclist{"FLAG"}}[$thisrow]);
                            # and the final flags 8 & 9 are merged into the
                            # summary values
                            $tmpflagpnt[$nr-1] = 
                                merge_flags($tmpflagpnt[$nr-1],
                                            substr(
                                                ${$srclist{"FLAG"}}[$thisrow],
                                                7,2)
                                );
			    # save maximum pile-up level
			    $tmppileup[$nr-1] = ${$srclist{"PILEUP"}}[$thisrow]
				if (defined(${$srclist{"PILEUP"}}[$thisrow]) &&
				    ${$srclist{"PILEUP"}}[$thisrow] < 1e99   &&
				    ${$srclist{"PILEUP"}}[$thisrow] > 0.0    &&
				    ${$srclist{"PILEUP"}}[$thisrow]
				    > $tmppileup[$nr-1]);
                        } else {
                            ${$srclist{"FLAG"}}[$thisrow] = " ";
                        }
                        # done.

                    } else {                     ### begin: instrument undefined
                        # set instrument-specific columns to NULL or zero, if
                        # instrument has not been defined; keep the others
                        ${$srclist{$_}}[$thisrow] = 0.0
			    for ("ONTIME"    );
                        ${$srclist{$_}}[$thisrow] = 
			    $mynulls{$suminfo{form}{$suminfo{namesum}{$_}}}
			    for ("SCTS"      ,
				 "SCTS_ERR"  ,
				 "DET_ML"    ,
				 "BG_MAP"    ,
				 "EXP_MAP"   ,
				 "FLUX"      ,
				 "FLUX_ERR"  ,
				 "RATE"      ,
				 "RATE_ERR"  ,
				 "RAWX"      ,
				 "RAWY"      ,
				 "OFFAX"     ,
				 "CCDNR"     ,
				 "HR1"       ,
				 "HR1_ERR"   ,
				 "HR2"       ,
				 "HR2_ERR"   ,
				 "HR3"       ,
				 "HR3_ERR"   ,
				 "HR4"       ,
				 "HR4_ERR"   ,
				 "PILEUP"    ,
				 "MASKFRAC"  ,
				 "EEF"       ,
				 "VIGNETTING");
                        # MASKFRAC zero means: flag1 T in pointing summary
                        if (${$srclist{"FLAG"}}[$thisrow] =~ /F|T/) {
                            # propagate flag1 to merged camera and summary flag
                            (substr $tmpflaginst,       0, 1) = "T";
                            (substr $tmpflagpnt[$nr-1], 0, 1) = "T";
                        } else {
                            ${$srclist{"FLAG"}}[$thisrow] = " ";
                        }

                    }                    ### end: instrument defined / undefined

                }                             ### end loop over new summary rows



#               ################################################################
                ## 2. set final column values in SUMMARY ROW PER POINTING
                #                                                    (="pntsum")

                # PREPARATORY WORK (on pointing rows without summary)

		# number of active instruments
		$sum_nexpos += $sum_ninst;

                # temporary value of EXP_MAP, used for checks
                my $tmpexpmap = 
                    List::Util::sum 
                    @{$srclist{"EXP_MAP"}}[@pntinds[$nnewrows..$#pntinds]];

                # calculate total fluxes and flux errors:
                # error-weighted mean of summary values per instrument;
		# same for rates;
                # offaxis angle: minimum of instrument values
                # ontime: maximum of instrument values;
                my $tmpsqerr;
                $tmpflux    = 0.;
                $tmpfluxerr = 0.;
                for my $rowind ($pntinds[0]+1..$pntinds[0]+$nnewrows-1) {
                    if (defined(${srclist{"FLUX"}}[$rowind])
                        && ${srclist{"FLUX"}}[$rowind] < 1e99
                        && ${srclist{"FLUX_ERR"}}[$rowind] !=0 ) {
                        $tmpsqerr = 
                            1.0
                            /${srclist{"FLUX_ERR"}}[$rowind]
                            /${srclist{"FLUX_ERR"}}[$rowind];
                        $tmpflux 
                            += ${srclist{"FLUX"}}[$rowind]*$tmpsqerr;
                        $tmpfluxerr += $tmpsqerr;
                    }
                }
                if ($tmpflux > 0) {
                    $tmpflux /= $tmpfluxerr;
                    $tmpfluxerr = 1.0/sqrt($tmpfluxerr);
                } else {
                    $tmpflux = $mynull;
                    $tmpfluxerr = $mynull;
                }

                # and up to four hardness ratios, averaged over instruments
                for my $ie (0..$nbands-2) {
                                    # loop over energy bands (max. $nband-1 HRs)
                    $tmphrval[$ie] = 0;
                    $tmphrerr[$ie] = 0; 
                    for my $ii (1..$nnewrows-1) {
                        if (defined 
                            ${srclist{"HR".($ie+1)}}[$pntinds[0]+$ii]
                            && 
                            defined 
                            ${srclist{"HR".($ie+1)."_ERR"}}[$pntinds[0]+$ii]
                            && 
                            ${srclist{"HR".($ie+1)."_ERR"}}[$pntinds[0]+$ii] > 0
			    && ${srclist{"HR".($ie+1)}}[$pntinds[0]+$ii]
			       < 1e99
			    && ${srclist{"HR".($ie+1)."_ERR"}}[$pntinds[0]+$ii]
			       < 1e99
                            ) {
                          $tmpsqerr = 1.0
                              /${srclist{"HR".($ie+1)."_ERR"}}[$pntinds[0]+$ii]
                              /${srclist{"HR".($ie+1)."_ERR"}}[$pntinds[0]+$ii];
                          $tmphrval[$ie] 
                              += ${srclist{"HR".($ie+1)}}[$pntinds[0]+$ii]
                              *$tmpsqerr;
                          $tmphrerr[$ie] += $tmpsqerr;
                        }
                    }
                    if (defined($tmphrval[$ie]) && $tmphrerr[$ie] > 0) {
                        $tmphrval[$ie] /= $tmphrerr[$ie];
                        $tmphrerr[$ie] = 1.0/sqrt($tmphrerr[$ie]);
                    } else {
                        $tmphrval[$ie] = $mynull;
                        $tmphrerr[$ie] = $mynull;
                    }
                }                                   # end loop over energy bands


                # COLUMNS ######################################################
                # SCTS: sum                                             (pntsum)
                ${$srclist{"SCTS"}}[$pntinds[0]] =
                    (List::Util::sum @{$srclist{"SCTS"}}
                                        [@pntinds[$nnewrows..$#pntinds]]);
                # SCTS_ERR: square root of quadratic sum                (pntsum)
                ${$srclist{"SCTS_ERR"}}[$pntinds[0]] =
                    (sqrt(List::Util::sum 
                          ( map {$_*$_} (@{$srclist{"SCTS_ERR"}}
                                            [@pntinds[$nnewrows..$#pntinds]]) )
                     ));
                # DET_ML: equivalent likelihood via gamma function or null
                # (above)
                # BG_MAP: sum; null if exp_map<=0                       (pntsum)
                ${$srclist{"BG_MAP"}}[$pntinds[0]] =
                    (defined($tmpexpmap) && $tmpexpmap>0)
                    ? (List::Util::sum @{$srclist{"BG_MAP"}}
                                          [@pntinds[$nnewrows..$#pntinds]])
                    : $mynull;
                # EXP_MAP: null in summary rows with ID_INST [and/or ID_BAND]==0
                ${$srclist{"EXP_MAP"}}[$pntinds[0]] = $mynull;
                # FLUX: error-weighted mean of per-instrument fluxes    (pntsum)
                ${$srclist{"FLUX"}}[$pntinds[0]] = $tmpflux;
                # FLUX_ERR: error of error-weighted mean flux (see above)
                ${$srclist{"FLUX_ERR"}}[$pntinds[0]] = $tmpfluxerr;
                # RATE: sum                                             (pntsum)
                ${$srclist{"RATE"}}[$pntinds[0]] =
                    (List::Util::sum @{$srclist{"RATE"}}
                                       [@pntinds[$nnewrows..$#pntinds]]);
                # RATE_ERR: square root of quadratic sum                (pntsum)
                ${$srclist{"RATE_ERR"}}[$pntinds[0]] =
                    (sqrt(List::Util::sum 
                          ( map {$_*$_} (@{$srclist{"RATE_ERR"}}
                                            [@pntinds[$nnewrows..$#pntinds]]) )
                     ));
                # differing from emldetect, all-band HRs are set
                # HR1: calculated above, set to null if lower than -10
                ${$srclist{"HR1"}}[$pntinds[0]] =
                    ($tmphrval[0] >= -10.0)
                    ? $tmphrval[0]
                    : $mynull;
                # HR1_ERR                                               (pntsum)
                ${$srclist{"HR1_ERR"}}[$pntinds[0]] =
                    ($tmphrerr[0] >= -10.0)
                    ? $tmphrerr[0]
                    : $mynull;
                # HR2                                                   (pntsum)
                ${$srclist{"HR2"}}[$pntinds[0]] =
                    ($tmphrval[1] >= -10.0)
                    ? $tmphrval[1]
                    : $mynull;
                # HR2_ERR                                               (pntsum)
                ${$srclist{"HR2_ERR"}}[$pntinds[0]] =
                    ($tmphrerr[1] >= -10.0)
                    ? $tmphrerr[1]
                    : $mynull;
                # HR3                                                   (pntsum)
                ${$srclist{"HR3"}}[$pntinds[0]] =
                    ($tmphrval[2] >= -10.0)
                    ? $tmphrval[2]
                    : $mynull;
                # HR3_ERR                                               (pntsum)
                ${$srclist{"HR3_ERR"}}[$pntinds[0]] =
                    ($tmphrerr[2] >= -10.0)
                    ? $tmphrerr[2]
                    : $mynull;
                # HR4                                                   (pntsum)
                ${$srclist{"HR4"}}[$pntinds[0]] =
                    ($tmphrval[3] >= -10.0)
                    ? $tmphrval[3]
                    : $mynull;
                # HR4_ERR                                               (pntsum)
                ${$srclist{"HR4_ERR"}}[$pntinds[0]] =
                    ($tmphrerr[3] >= -10.0)
                    ? $tmphrerr[3]
                    : $mynull;
                # OFFAX: smallest value of the EPIC cameras (cf. 3XMM)
                ${$srclist{"OFFAX"     }}[$pntinds[0]] = List::Util::min
                    @{$srclist{"OFFAX" }}[@pntinds[(1..$nnewrows-1)]];
                # ONTIME: largest value of the EPIC cameras (cf. 3XMM)
                ${$srclist{"ONTIME"    }}[$pntinds[0]] = List::Util::max
                    @{$srclist{"ONTIME"}}[@pntinds[(1..$nnewrows-1)]];
		# PILEUP: undefined                                     (pntsum)
		${$srclist{"PILEUP"    }}[$pntinds[0]] = $mynull;
                # FLAG: replace by merged camera flags - if applicable
                ${$srclist{"FLAG"      }}[$pntinds[0]] = 
                    (${$srclist{"FLAG"}}[$pntinds[0]] =~ /F|T/)
                    ? $tmpflaginst
                    : " ";
                # following columns are null in INST_ID==0 summary rows:
		${$srclist{$_}}[$pntinds[0]] = 
		    $mynulls{$suminfo{form}{$suminfo{namesum}{$_}}}
		for ("RAWX"    ,
		     "RAWY"      ,
		     "CCDNR"     ,
		     "MASKFRAC"  ,
		     "EEF"       ,
		     "VIGNETTING");
                # values only for the catalogue-like rows:
                ${$srclist{"MJD_FIRST"}}[$pntinds[0]] =
                    (defined $hhead{sprintf ("%010i",$instidpnt)}{"MJD_FIRST"})
                    ? $hhead{sprintf ("%010i",$instidpnt)}{"MJD_FIRST"}
                    : $mynull;
                ${$srclist{"MJD_LAST"}}[$pntinds[0]] =
                    (defined $hhead{sprintf ("%010i",$instidpnt)}{"MJD_LAST"})
                    ? $hhead{sprintf ("%010i",$instidpnt)}{"MJD_LAST"}
                    : $mynull;

                ################################################################
                ## insert the column values into the catalogue-like row
                # (use index because of issue with unshift)
                $scnt += 1;
                # general information                                 (pointing)
                $sumlist[$scnt]{"SRCID"    } = $sum_srcid;
                $sumlist[$scnt]{"OBS_ID"   } = sprintf("%010i",$instidpnt);
                $sumlist[$scnt]{"N_OBS"    } = $mynulls{"J"};
                $sumlist[$scnt]{"N_CONTRIB"} = $mynulls{"J"};
                $sumlist[$scnt]{"N_EXP"    } = $sum_ninst;
                $sumlist[$scnt]{"OVERLAP"  } = ($sum_ncontrib > 1) ? 1 : 0;
                $sumlist[$scnt]{"DIST_REF" } = $dist_ref;
		$sumlist[$scnt]{"N_BLEND"  } = $tmpnblend[$srcid];
		$sumlist[$scnt]{$_} = 
                    ${$srclist{$suminfo{emlname}{$_}}}[$pntinds[0]]
                    for ("ML_ID_SRC"    ,
                         "BOX_ID_SRC"   ,
                         "RA"           ,
                         "DEC"          ,
                         "RADEC_ERR"    ,
                         "LII"          ,
                         "BII"          ,
                         "X_IMA"        ,
                         "X_IMA_ERR"    ,
                         "Y_IMA"        ,
                         "Y_IMA_ERR"    ,
                         "DIST_NN"      ,
                         "EXTENT"       ,
                         "EXTENT_ERR"   ,
                         "EXTENT_ML"    ,
                         "MJD_FIRST"    ,
                         "MJD_LAST"     );
                # band summary values taken from summary rows per pointing
                for my $ii (0..$#insts) {
                    $sumlist[$scnt]{$insts[$ii]."_".$_} = 
                        ${$srclist{$suminfo{emlname}{$_}}}[$pntinds[0]+$ii]
                        for ("FLUX"      ,
                             "FLUX_ERR"  ,
                             "RATE"      ,
                             "RATE_ERR"  ,
                             "CTS"       ,
                             "CTS_ERR"   ,
                             "DET_ML"    ,
                             "HR1"       ,
                             "HR1_ERR"   ,
                             "HR2"       ,
                             "HR2_ERR"   ,
                             "HR3"       ,
                             "HR3_ERR"   ,
                             "HR4"       ,
                             "HR4_ERR"   ,
                             "EXP"       ,
                             "BG"        ,
                             "ONTIME"    ,
                             "PILEUP"    ,
                             "MASKFRAC"  ,
                             "OFFAX"     ,
                             "VIG"       ,
                             "FLAG"      );
                }
                # calculate EPIC band values of the flux              (pointing)
                for my $ie (1..$nbands) {             # loop over energy bands
                    # EPIC fluxes: weighted means of instrument values
                    my $tmpsqerr;
                    $tmpflux = 0.;
                    $tmpfluxerr = 0.;
                    for my $pntind (@pntinds[(grep
                              { ${$srclist{"ID_INST"}}[$pntinds[$_]]
                                    =~ "$instidpnt"
                             && ${$srclist{"ID_BAND"}}[$pntinds[$_]] eq $ie    }
                              0..$#pntinds)]) {
                        if (defined(${srclist{"FLUX"}}[$pntind])
                            && ${srclist{"FLUX"}}[$pntind] < 1e99
                            && ${srclist{"FLUX_ERR"}}[$pntind] !=0 ) {
                            $tmpsqerr = 1.0/${srclist{"FLUX_ERR"}}[$pntind]
                                /${srclist{"FLUX_ERR"}}[$pntind];
                            $tmpflux 
                                += ${srclist{"FLUX"}}[$pntind]*$tmpsqerr;
                            $tmpfluxerr += $tmpsqerr;
                        }
                    }
                    if ($tmpfluxerr > 0) {
                        $tmpflux /= $tmpfluxerr;
                        $tmpfluxerr = 1.0/sqrt($tmpfluxerr);
                    } else {
                        $tmpflux = $mynull;
                        $tmpfluxerr = 0.0;
                    }
                    # copy to source list
                    $sumlist[$scnt]{"EP_".($ie)."_FLUX"}     = $tmpflux;
                    $sumlist[$scnt]{"EP_".($ie)."_FLUX_ERR"} = $tmpfluxerr;
                }                                   # end loop over energy bands
                # copy instrument band values taken from "raw" rows per pointing
                for my $ii (1..$#insts) {
                    for my $ib (1..$nbands) {
                        my $eind =
                            @pntinds[(grep
                                { ${$srclist{"ID_INST"}}[$pntinds[$_]] 
                                      eq "$instidpnt"."$ii"
                                  && ${$srclist{"ID_BAND"}}[$pntinds[$_]]
                                      eq $ib                             }
                                      0..$#pntinds)];
                        # instrument may be empty
                        if (defined $eind) {
                            $sumlist[$scnt]{$insts[$ii]."_".$ib."_".$_} = 
                                ${$srclist{$suminfo{emlname}{$_}}}[$eind]
                                for ("FLUX"      ,
                                     "FLUX_ERR"  ,
                                     "RATE"      ,
                                     "RATE_ERR"  ,
				     "CTS"      ,
				     "CTS_ERR"  ,
                                     "DET_ML"    ,
                                     "EXP"       ,
                                     "BG"        ,
                                     "VIG"       );
			} else {
			    # set to null explicitly, if instrument not defined
                            $sumlist[$scnt]{$insts[$ii]."_".$ib."_".$_} = 
                                $mynull
                                for ("FLUX"      ,
                                     "FLUX_ERR"  ,
                                     "RATE"      ,
                                     "RATE_ERR"  ,
                                     "DET_ML"    ,
                                     "EXP"       ,
                                     "BG"        ,
                                     "VIG"       );
                        }
			# and if maskfrac below threshold
			if (! defined $sumlist[$scnt]{$insts[$ii]."_MASKFRAC"}
			    || $sumlist[$scnt]{$insts[$ii]."_MASKFRAC"}<$maskthr
			    || ! defined
			    $sumlist[$scnt]{$insts[$ii]."_".$ib."_EXP"} ||
			    $sumlist[$scnt]{$insts[$ii]."_".$ib."_EXP"}<=0
			    ) {
			    $sumlist[$scnt]{$insts[$ii]."_".$ib."_".$_} = 
				$mynull
				for ("FLUX"     ,
				     "FLUX_ERR" ,
				     "RATE"     ,
				     "RATE_ERR" ,
				     "CTS"      ,
				     "CTS_ERR"  ,
				     "DET_ML"   ); 
			}
			if (! defined
			    $sumlist[$scnt]{$insts[$ii]."_".$ib."_EXP"} ||
			    $sumlist[$scnt]{$insts[$ii]."_".$ib."_EXP"}<=0 
			    ) {
			    $sumlist[$scnt]{$insts[$ii]."_".$ib."_".$_} = 
				$mynull
				for ("EXP",
				     "BG" );
			}
                    }
		    # and for the instrument values
		    if (! defined $sumlist[$scnt]{$insts[$ii]."_MASKFRAC"}
			|| $sumlist[$scnt]{$insts[$ii]."_MASKFRAC"}<$maskthr
			|| ! defined $sumlist[$scnt]{$insts[$ii]."_EXP"}
			|| $sumlist[$scnt]{$insts[$ii]."_EXP"}<=0
			) {
			$sumlist[$scnt]{$insts[$ii]."_".$_} = 
			    $mynull
			    for ("FLUX"      ,
				 "FLUX_ERR"  ,
				 "RATE"      ,
				 "RATE_ERR"  ,
				 "CTS"       ,
				 "CTS_ERR"   ,
				 "DET_ML"    );
		    }
		    if (! defined $sumlist[$scnt]{$insts[$ii]."_EXP"}
			|| $sumlist[$scnt]{$insts[$ii]."_EXP"}<=0
			) {
			$sumlist[$scnt]{$insts[$ii]."_".$_} = 
			    $mynull
			    for ("EXP",
				 "BG" );
		    }
                }
		# currently no variability information for pointings
		$sumlist[$scnt]{"VAR_CHI2"}    = $mynull;
		$sumlist[$scnt]{"VAR_PROB"}    = $mynull;
		$sumlist[$scnt]{"FRATIO"}     = $mynull;
		$sumlist[$scnt]{"FRATIO_ERR"} = $mynull;
		$sumlist[$scnt]{"FLUXVAR"}    = $mynull;
		for my $ib (1..$nbands) {
		    $sumlist[$scnt]{"VAR_CHI2_".($ib)}      = $mynull;
		    $sumlist[$scnt]{"VAR_PROB_".($ib)}      = $mynull;
		    $sumlist[$scnt]{"FRATIO_".($ib)}        = $mynull;
		    $sumlist[$scnt]{"FRATIO_".($ib)."_ERR"} = $mynull;
		    $sumlist[$scnt]{"FLUXVAR_".($ib)}       = $mynull;
		}
                # remaining pointing information (strings & numbers separately)
                for ("REVOLUT"   ,
		     "PA_PNT"    ,
		     "PN_SUBMODE",
                     "M1_SUBMODE",
                     "M2_SUBMODE",
                     "PN_FILTER" ,
                     "M1_FILTER" ,
                     "M2_FILTER" ) {
                    $sumlist[$scnt]{$_} = 
                        (defined $hhead{$sumlist[$scnt]{"OBS_ID"}}{$_})
                        ? $hhead{$sumlist[$scnt]{"OBS_ID"}}{$_}
                        : $mynulls{substr $suminfo{form}{$_},-1};
		}
                # STACK_FLAG: integer representation of EP_FLAG       (pointing)
                if ($sumlist[$scnt]{"EP_FLAG"} =~ /F|T/) {
                    # 0, if all flags are "F"
                    $sumlist[$scnt]{"STACK_FLAG"} = 0;
                    # 1, if any of flags 1, 2, 3, 9 is / are true for a camera
                    #    (~ close to a bad area)
                    $sumlist[$scnt]{"STACK_FLAG"} = 1
                        if (substr($sumlist[$scnt]{"EP_FLAG"}, 0, 3) =~ "T"
                            || substr($sumlist[$scnt]{"EP_FLAG"}, 8, 1) =~ "T");
                    # 2, if flag 7 (any of flags 4-6) or 8 are true
                    #    (~ probably spurious)
                    $sumlist[$scnt]{"STACK_FLAG"} = 2
                        if (substr($sumlist[$scnt]{"EP_FLAG"}, 6, 2) =~ "T");
                    ${$srclist{"STACK_FLAG"}}[$srcinds[0]] += 1
                        if $sumlist[$scnt]{"STACK_FLAG"} == 2;
                                             # redundant if - I know
                } else {
                    $sumlist[$scnt]{"STACK_FLAG"} = $mynull;
                }

                # done.

            }                                         ### end: rows per pointing
        }                                            ### end loop over pointings



        ########################################################################
        ## MODIFY MAJOR ROWS (total summary last)

        ## number of instruments: FIXED
        for my $nr (reverse(1..3)) {                   ### loop over instruments
            # indices of pointing rows corresponding to this instrument number:
            # @{$instrows{$nr}}

            # initialize (empty) arrays of mask fractions, fluxes and errors
            my @instmaskfracs = ();
            my @instfluxvals = ();
            my @instfluxerrs = ();
            my @instratevals = ();
            my @instrateerrs = ();

            # rows holding the instrument summary per energy band
            my @isumrows = @srcinds[(grep
                         { "0"."${$srclist{'ID_INST'}}[$srcinds[$_]]" == $nr }
                                     0..$#srcinds)];

            # add to list of valid rows
            unshift @srcvalinds, @isumrows;


            # all rows of this detections which have ID_INST == $nr
            if ($#{$instrows{$nr}} >= 0) {              # if instrows{$nr} found
                for my $ir (reverse(@isumrows)) {  ### loop over instrument rows

#                   ############################################################
                    ## I. PER INSTRUMENT PER ENERGY BAND             (="majene")
                                                             # begin ID_BAND !=0
                    if ("0".${$srclist{"ID_BAND"}}[$ir] != 0) {

                        # indices of pointing rows with same ID_BAND
                        my @pntrows = 
                            @{$instrows{$nr}}[(grep
                                { ${$srclist{"ID_BAND"}}[${$instrows{$nr}}[$_]]
                                      == ${$srclist{"ID_BAND"}}[$ir] }
                                               0..$#{$instrows{$nr}})];
                        # temporary value of EXP_MAP, used for checks
                        $tmpexpmap 
                            = List::Util::sum
                                   ( map {(defined($_) && $_ < 1e33) ? $_ : 0.0}
                                     (@{$srclist{"EXP_MAP"}}[@pntrows]) );
                        # temporary value of MASKFRAC: maximum of pointings
                        $tmpmaskfrac 
                            = List::Util::max
                                   ( map {(defined($_) && $_ < 1e33) ? $_ : 0.0}
                                     (@{$srclist{"MASKFRAC"}}[@pntrows]) );
                        push @instmaskfracs, $tmpmaskfrac;
                        # temporary values of flux, flux error:
                        # needed for summary row
                        $tmpflux = 
                            (defined($tmpexpmap) && $tmpexpmap>0)
                            ? 
                            (List::Util::sum 
                             ( map { ( ${$srclist{"EXP_MAP"}}[$_] > 0)
                                         ? ${$srclist{"FLUX"}}[$_]
                                         *${$srclist{"EXP_MAP"}}[$_]
                                         : 0.0 } @pntrows ))
                            / $tmpexpmap
                            : 
                            $mynull;
                        push @instfluxvals, $tmpflux;
                        $tmpfluxerr = 
                            (defined($tmpexpmap) && $tmpexpmap>0)
                            ?
                            sqrt(List::Util::sum 
                                 ( map { ( ${$srclist{"EXP_MAP"}}[$_] > 0)
                                             ? ${$srclist{"FLUX_ERR"}}[$_]
                                             *${$srclist{"FLUX_ERR"}}[$_]
                                             *${$srclist{"EXP_MAP"}}[$_]
                                             *${$srclist{"EXP_MAP"}}[$_] 
                                             : 0.0 }
                                   @pntrows )) / $tmpexpmap
                                       : 
                                       $mynull;
                        push @instfluxerrs, $tmpfluxerr;
                        $tmprate = 
                            (defined($tmpexpmap) && $tmpexpmap>0)
                            ? 
                            (List::Util::sum 
                             ( map { ( ${$srclist{"EXP_MAP"}}[$_] > 0)
                                         ? ${$srclist{"RATE"}}[$_]
                                         *${$srclist{"EXP_MAP"}}[$_]
                                         : 0.0 } @pntrows ))
                            / $tmpexpmap
                            : 
                            $mynull;
                        push @instratevals, $tmprate;
                        $tmprateerr = 
                            (defined($tmpexpmap) && $tmpexpmap>0)
                            ?
                            sqrt(List::Util::sum 
                                 ( map { ( ${$srclist{"EXP_MAP"}}[$_] > 0)
                                             ? ${$srclist{"RATE_ERR"}}[$_]
                                             *${$srclist{"RATE_ERR"}}[$_]
                                             *${$srclist{"EXP_MAP"}}[$_]
                                             *${$srclist{"EXP_MAP"}}[$_] 
                                             : 0.0 }
                                   @pntrows )) / $tmpexpmap
                                       : 
                                       $mynull;
                        push @instrateerrs, $tmprateerr;


                        # COLUMNS ##############################################

                        # SCTS: sum; null if exp_map<=0 or maskfrac<thresh.
                        ${$srclist{"SCTS"}}[$ir] =
                            (defined($tmpmaskfrac) && $tmpmaskfrac>=$maskthr &&
                             defined($tmpexpmap) && $tmpexpmap>0)
                            ? (List::Util::sum @{$srclist{"SCTS"}}[@pntrows])
                            : $mynull;
                        # SCTS_ERR: square root of quadratic sum or null
                        ${$srclist{"SCTS_ERR"}}[$ir] =
                            (defined($tmpmaskfrac) && $tmpmaskfrac>=$maskthr &&
                             defined($tmpexpmap) && $tmpexpmap>0)
                            ? (sqrt(List::Util::sum 
                                    ( map {(defined($_)) ? $_*$_ : 0.0}
                                      (@{$srclist{"SCTS_ERR"}}[@pntrows]) ) ))
                            : $mynull;
                        # DET_ML: equivalent likelihood via gamma function 
                        #         or NULL
                        ${$srclist{"DET_ML"}}[$ir] =
                            (defined($tmpmaskfrac) && $tmpmaskfrac>=$maskthr &&
                             defined($tmpexpmap) && $tmpexpmap>0)
                            ? likelihood(   # dofs/2, sum(likelihoods)
                               (2+(${$srclist{"EXT"}}[$pntrows[0]]>0)
                                +(scalar @pntrows))/2.0,
                               List::Util::sum @{$srclist{"DET_ML"}}[@pntrows] )
                            : $mynull;
                        # BG_MAP: sum; null if exp_map<=0               (majene)
                        ${$srclist{"BG_MAP"}}[$ir] =
                            (defined($tmpexpmap) && $tmpexpmap>0)
                            ? (List::Util::sum @{$srclist{"BG_MAP"}}[@pntrows])
                            : $mynull;
                        # EXP_MAP: sum; null if negative                (majene)
                        ${$srclist{"EXP_MAP"}}[$ir] =
                            (defined($tmpexpmap) && $tmpexpmap>0)
                            ? $tmpexpmap
                            : $mynull;
                        # FLUX: exposure-weighted sum;                  (majene)
                        #       null if exp_map<=0 or m.frac<thr.
                        ${$srclist{"FLUX"}}[$ir] =
                            (defined($tmpmaskfrac) && $tmpmaskfrac>=$maskthr &&
                             defined($tmpexpmap) && $tmpexpmap>0)
                            ? $tmpflux
                            : $mynull;
                        # FLUX_ERR: exposure-weighted square root of    (majene)
                        #           quadratic sum or null
                        ${$srclist{"FLUX_ERR"}}[$ir] =
                            (defined($tmpmaskfrac) && $tmpmaskfrac>=$maskthr &&
                             defined($tmpexpmap) && $tmpexpmap>0)
                            ? $tmpfluxerr
                            : $mynull;
                        # RATE: sum; null if exp_map<=0 or maskfrac<thresh.
                        ${$srclist{"RATE"}}[$ir] =
                            (defined($tmpmaskfrac) && $tmpmaskfrac>=$maskthr &&
                             defined($tmpexpmap) && $tmpexpmap>0)
                            ? $tmprate
                            : $mynull;
                        # RATE_ERR: square root of quadratic sum or null
                        ${$srclist{"RATE_ERR"}}[$ir] =
                            (defined($tmpmaskfrac) && $tmpmaskfrac>=$maskthr &&
                             defined($tmpexpmap) && $tmpexpmap>0)
                            ? $tmprateerr
                            : $mynull;
                        # RAWX, RAWY, OFFAX, CCDNR: null (pointing-dependent)
                        ${$srclist{"RAWX" }}[$ir] = $mynulls{"J"};
                        ${$srclist{"RAWY" }}[$ir] = $mynulls{"J"};
                        ${$srclist{"OFFAX"}}[$ir] = $mynulls{"E"};
                        ${$srclist{"CCDNR"}}[$ir] = $mynulls{"J"};
                        # HR?, HR?_ERR: unchanged (null)                (majene)
                        #
                        # MASKFRAC: maximum of pointings                (majene)
                        ${$srclist{"MASKFRAC"}}[$ir] = $tmpmaskfrac;
                        # EEF: unchanged                                (majene)
                        #
                        # VIGNETTING: NULL                              (majene)
                        ${$srclist{"VIGNETTING"}}[$ir] = $mynull;
                        # ONTIME: set to zero if negative               (majene)
                        ${$srclist{"ONTIME"    }}[$ir] = 0.0 
                            if (${$srclist{"ONTIME"}}[$ir] < 0.0);
                        # PILEUP: undefined                             (majene)
                        ${$srclist{"PILEUP"    }}[$ir] = $mynull ;
                        # FLAG: flags 1-7 from dpssflag; 8-9 from pointing rows
                        ${$srclist{"FLAG"}}[$ir] = 
                            (${$srclist{"FLAG"}}[$ir] =~ /F|T/)
                            ? (substr ${$srclist{"FLAG"}}[$ir],0,7)
                              .$tmpflagband[$nr.${$srclist{"ID_BAND"}}[$ir]]
                            : " ";


#                   ############################################################
                    ## II. SUMMARY ROW PER INSTRUMENT                (="majins")
                    } else {                                        # ID_BAND==0

                        # temporary value of MASKFRAC: minimum of maxima
                        $tmpmaskfrac = List::Util::min @instmaskfracs;

                        # COLUMNS ##############################################

                        # SCTS: sum; null if exp_map<=0 or maskfrac<thresh.
                        ${$srclist{"SCTS"}}[$ir] =
                            (defined($tmpmaskfrac) && $tmpmaskfrac>=$maskthr &&
                             defined($tmpexpmap) && $tmpexpmap>0)
                            ? (List::Util::sum 
                               @{$srclist{"SCTS"}}[@{$instrows{$nr}}])
                            : $mynull;
                        # SCTS_ERR: square root of quadratic sum or null
                        ${$srclist{"SCTS_ERR"}}[$ir] =
                            (defined($tmpmaskfrac) && $tmpmaskfrac>=$maskthr &&
                             defined($tmpexpmap) && $tmpexpmap>0)
                            ? (sqrt(List::Util::sum 
                                  ( map {(defined($_)) ? $_*$_ : 0.0}
                                    (@{$srclist{"SCTS_ERR"}}[@{$instrows{$nr}}])
                                  ) ))
                            : $mynull;
                        # DET_ML: equivalent likelihood via gamma function
                        ${$srclist{"DET_ML"}}[$ir] =
                            (defined($tmpmaskfrac) && $tmpmaskfrac>=$maskthr &&
                             defined($tmpexpmap) && $tmpexpmap>0)
                            ? likelihood(   # dofs/2, sum(likelihoods)
                              (2+(${$srclist{"EXT"}}[${$instrows{$nr}}[0]]>0)
                               +(scalar @{$instrows{$nr}}))/2.0,
                               List::Util::sum 
                               @{$srclist{"DET_ML"}}[@{$instrows{$nr}}] )
                            : $mynull;
                        # BG_MAP: sum; null if exp_map<=0               (majins)
                        ${$srclist{"BG_MAP"}}[$ir] =
                            (defined($tmpexpmap) && $tmpexpmap>0)
                            ? (List::Util::sum
                               @{$srclist{"BG_MAP"}}[@{$instrows{$nr}}])
                            : $mynull;
                        # EXP_MAP: unchanged (null)                     (majins)
                        #
                        # FLUX: exposure-weighted sum; null if exp_map<=0 
                        #       or maskfrac<threshold [later-on]
                        ${$srclist{"FLUX"}}[$ir] = 
                            List::Util::sum
                                 ( map {(defined($_)) ? $_ : 0.0}
                                   (@instfluxvals) ) ;
                        # FLUX_ERR: exposure-weighted square root of 
                        #           quadratic sum or null               (majins)
                        ${$srclist{"FLUX_ERR"}}[$ir] = 
                            sqrt(List::Util::sum 
                                 ( map {(defined($_) && $_<1e99) ? $_*$_ : 0.0}
                                   (@instfluxerrs)              ) );
                        # RATE: sum; null if exp_map<=0 or maskfrac<thresh.
                        ${$srclist{"RATE"}}[$ir] =
                            List::Util::sum
                                 ( map {(defined($_)) ? $_ : 0.0}
                                   (@instratevals) ) ;
                        # RATE_ERR: square root of quadratic sum or null
                        ${$srclist{"RATE_ERR"}}[$ir] =
                            sqrt(List::Util::sum 
                                 ( map {(defined($_) && $_<1e99) ? $_*$_ : 0.0}
                                   (@instrateerrs)              ) );
                        # RAWX, RAWY, OFFAX, CCDNR: null in summary rows
                        ${$srclist{"RAWX" }}[$ir] = $mynulls{"J"};
                        ${$srclist{"RAWY" }}[$ir] = $mynulls{"J"};
                        ${$srclist{"OFFAX"}}[$ir] = $mynulls{"E"};
                        ${$srclist{"CCDNR"}}[$ir] = $mynulls{"J"};
                        # HR?, HR?_ERR: unchanged
                        #
                        # MASKFRAC: minimum of maxima (or vice versa ;-) )
                        ${$srclist{"MASKFRAC"}}[$ir] = $tmpmaskfrac;
                        # EEF: unchanged
                        #
                        # VIGNETTING: NULL                              (majins)
                        ${$srclist{"VIGNETTING"}}[$ir] = $mynull;
                        # ONTIME: sum of pointing values                (majins)
                        ${$srclist{"ONTIME"    }}[$ir] = $tmpontime[$nr-1];
			# PILEUP: maximum of defined values from pointing rows
                        ${$srclist{"PILEUP"    }}[$ir] = $tmppileup[$nr-1];
                        # FLAG: flags 1-7 from dpssflag; 8-9 from pointing rows
                        ${$srclist{"FLAG"}}[$ir] = 
                            (${$srclist{"FLAG"}}[$ir] =~ /F|T/)
                            ? (substr ${$srclist{"FLAG"}}[$ir],0,7)
                              .$tmpflagpnt[$nr-1]
                            : " ";
                    }                                              # end ID_BAND

                }                              ### end loop over instrument rows

            } else {                               # end  if instrows{$nr} found
                ## instrument rows without detections:
                # set RAWX, RAWY, OFFAX, CCDNR to null; keep flag
                @rowinds = @srcinds[(grep
                           { "0".${$srclist{"ID_INST"}}[$srcinds[$_]] == $nr }
                                      (0..$#srcinds) ) ];
                @newvals = ($mynulls{"J"})x($nbands+1);
                @{$srclist{"RAWX" }}[@rowinds] = @newvals;
                @{$srclist{"RAWY" }}[@rowinds] = @newvals;
                @{$srclist{"CCDNR"}}[@rowinds] = @newvals;
                @newvals = ($mynull)x($nbands+1);
                @{$srclist{"OFFAX"}}[@rowinds] = @newvals;
                @{$srclist{"FLAG" }}[$_] = 
                    substr(${$srclist{"FLAG"}}[$thisrow], 0, 9)
                    for (@rowinds);
            }
                    
        }                                          ### end loop over instruments


#       ########################################################################
        ## III. TOTAL SUMMARY ROW                                    (="majsum")
        ## updating FLUX, HRs, MASKFRAC, ONTIME, FLAG

        unshift @srcvalinds, $srcinds[0];

        # indices of instrument summary rows
        @rowinds =
            @srcinds[(grep { ${$srclist{"ID_INST"}}[$srcinds[$_]] > 0
                                 && ${$srclist{"ID_INST"}}[$srcinds[$_]] < 4
                                 && ${$srclist{"ID_BAND"}}[$srcinds[$_]] eq 0 }
                      0..$#srcinds)];
        # indices of pointing summary rows
        @pntinds =
            @srcinds[(grep { ${$srclist{"ID_INST"}}[$srcinds[$_]] > 4
                         && substr(${$srclist{"ID_INST"}}[$srcinds[$_]],-1) eq 0
                         && ${$srclist{"ID_BAND"}}[$srcinds[$_]] eq 0 }
                      0..$#srcinds)];

        # calculate total FLUXes and FLUX_ERRors:                       (majsum)
        # sum of summary values per instrument weighted by errors;
        my $tmpsqerr;
        $tmpflux    = 0.;
        $tmpfluxerr = 0.;
        for my $rowind (@rowinds) {
            if (defined(${srclist{"FLUX"}}[$rowind])
                && ${srclist{"FLUX"}}[$rowind] < 1e99
                && ${srclist{"FLUX_ERR"}}[$rowind] !=0 ) {
                $tmpsqerr = 1.0/${srclist{"FLUX_ERR"}}[$rowind]
                    /${srclist{"FLUX_ERR"}}[$rowind];
                $tmpflux 
                    += ${srclist{"FLUX"}}[$rowind]*$tmpsqerr;
                $tmpfluxerr += $tmpsqerr;
            }
        }
        if ($tmpflux > 0) {
            $tmpflux /= $tmpfluxerr;
            $tmpfluxerr = 1.0/sqrt($tmpfluxerr);
        } else {
            $tmpflux = $mynull;
            $tmpfluxerr = 0.0;
        }
        ${$srclist{"FLUX"}}    [$srcinds[0]] = $tmpflux;
        ${$srclist{"FLUX_ERR"}}[$srcinds[0]] = $tmpfluxerr;

        # and up to four averaged HARDNESS RATIOs                       (majsum)
        for my $ie (0..$nbands-1) {                     # loop over energy bands
            $tmphrval[$ie] = 0;
            $tmphrerr[$ie] = 0; 
            for my $rowind (@rowinds) {
                if (defined ${srclist{"HR".($ie+1)}}[$rowind]
                    && defined ${srclist{"HR".($ie+1)."_ERR"}}[$rowind]
                    && ${srclist{"HR".($ie+1)."_ERR"}}[$rowind] > 0
                    && ${srclist{"HR".($ie+1)."_ERR"}}[$rowind] < 1e99
                    ) {
                    $tmpsqerr = 1.0/${srclist{"HR".($ie+1)."_ERR"}}[$rowind]
                        /${srclist{"HR".($ie+1)."_ERR"}}[$rowind];
                    $tmphrval[$ie] 
                        += ${srclist{"HR".($ie+1)}}[$rowind]*$tmpsqerr;
                    $tmphrerr[$ie] += $tmpsqerr;
                }
            }
            if (defined($tmphrval[$ie]) && $tmphrerr[$ie] > 0) {
                $tmphrval[$ie] /= $tmphrerr[$ie];
                $tmphrerr[$ie] = 1.0/sqrt($tmphrerr[$ie]);
            } else {
                $tmphrval[$ie] = $mynull;
                $tmphrerr[$ie] = $mynull;
            }
            # copy and set to null if lower than -10
            ${$srclist{"HR".($ie+1)}}[$srcinds[0]] =
                ($tmphrval[$ie] >= -10.0)
                ? $tmphrval[$ie]
                : $mynull;
            ${$srclist{"HR".($ie+1)."_ERR"}}[$srcinds[0]] =
                ($tmphrerr[$ie] >= -10.0)
                ? $tmphrerr[$ie]
                : $mynull;
        }                                           # end loop over energy bands

        # MASKFRAC: maximum of pointing values                          (majsum)
        ${$srclist{"MASKFRAC"}}[$srcinds[0]] = 
            List::Util::max ( map {(defined($_)) ? $_ : 0.0} 
                              (@{$srclist{"MASKFRAC"}}[@pntinds]) );

        # ONTIME: sum of pointing values                                (majsum)
        ${$srclist{"ONTIME"}}[$srcinds[0]] = 0.;
        for my $pntind (@pntinds) {
            ${$srclist{"ONTIME"}}[$srcinds[0]] +=
                ${$srclist{"ONTIME"}}[$pntind]
                if (defined ${$srclist{"ONTIME"}}[$pntind]);
        }

        # FLAG: only flags 1-9                                          (majsum)
        ${$srclist{"FLAG"}}[$srcinds[0]] = 
            (${$srclist{"FLAG"}}[$srcinds[0]] =~ /F|T/)
            ? substr ${$srclist{"FLAG"}}[$srcinds[0]],0,9
            : " ";


        ################################################################
        ## insert column values into the catalogue-like row
        $scnt += 1;
	# indices of pointings are $scnt minus number of contributing
	# pointings (sumlist is reversed)
	my @tpinds = ($scnt-$sum_ncontrib..$scnt-1);
        $sumlist[$scnt]{"SRCID"    } = $sum_srcid;
        $sumlist[$scnt]{"OBS_ID"   } = " ";
        $sumlist[$scnt]{"N_OBS"    } = $sum_nobs;
        $sumlist[$scnt]{"N_CONTRIB"} = $sum_ncontrib;
        $sumlist[$scnt]{"N_EXP"    } = $sum_nexpos;
	$sumlist[$scnt]{"OVERLAP"  } = ($sum_ncontrib > 1) ? 1 : 0;
	$sumlist[$scnt]{"DIST_REF" } = $dist_ref;
	$sumlist[$scnt]{"N_BLEND"  } = $tmpnblend[$srcid];
        # copy all-EPIC values
        $sumlist[$scnt]{$_} = ${$srclist{$suminfo{emlname}{$_}}}[$srcinds[0]]
            for ("ML_ID_SRC"    ,
                 "BOX_ID_SRC"   ,
                 "RA"           ,
                 "DEC"          ,
                 "RADEC_ERR"    ,
                 "LII"          ,
                 "BII"          ,
                 "X_IMA"        ,
                 "X_IMA_ERR"    ,
                 "Y_IMA"        ,
                 "Y_IMA_ERR"    ,
                 "DIST_NN"      ,
                 "EXTENT"       ,
                 "EXTENT_ERR"   ,
                 "EXTENT_ML"    );
        # copy all-EPIC and instrument values
        for my $ii (0..$#insts) {
            my $eind =
                @srcinds[(grep
                          { ${$srclist{"ID_INST"}}[$srcinds[$_]] eq $ii
                         && ${$srclist{"ID_BAND"}}[$srcinds[$_]] eq  0    }
                          0..$#srcinds)];
            if (defined $eind) {
            $sumlist[$scnt]{$insts[$ii]."_".$_} = 
                ${$srclist{$suminfo{emlname}{$_}}}[$eind]
                for ("FLUX"      ,
                     "FLUX_ERR"  ,
                     "RATE"      ,
                     "RATE_ERR"  ,
                     "CTS"       ,
                     "CTS_ERR"   ,
                     "DET_ML"    ,
                     "HR1"       ,
                     "HR1_ERR"   ,
                     "HR2"       ,
                     "HR2_ERR"   ,
                     "HR3"       ,
                     "HR3_ERR"   ,
                     "HR4"       ,
                     "HR4_ERR"   ,
                     "EXP"       ,
                     "BG"        ,
                     "ONTIME"    ,
                     "PILEUP"    ,
                     "MASKFRAC"  ,
                     "OFFAX"     ,
                     "VIG"       ,
                     "FLAG"      );
            } else {
            $sumlist[$scnt]{$insts[$ii]."_".$_} = $mynull
                for ("FLUX"      ,
                     "FLUX_ERR"  ,
                     "RATE"      ,
                     "RATE_ERR"  ,
                     "CTS"       ,
                     "CTS_ERR"   ,
                     "DET_ML"    ,
                     "HR1"       ,
                     "HR1_ERR"   ,
                     "HR2"       ,
                     "HR2_ERR"   ,
                     "HR3"       ,
                     "HR3_ERR"   ,
                     "HR4"       ,
                     "HR4_ERR"   ,
                     "EXP"       ,
                     "BG"        ,
                     "ONTIME"    ,
                     "PILEUP"    ,
                     "MASKFRAC"  ,
                     "OFFAX"     ,
                     "VIG"       );
            }
        }

        ## VAR_CHI2*: measure for flux variability, if N_CONTRIB >= 2
        if ($sum_ncontrib > 1
            && defined($tmpflux) && $tmpflux>0 && $tmpflux<1e99) {
            %fhash = ();
            $fhash{"meanflux"} = $tmpflux;
            ## all pointings
            @{$fhash{"fluxes"}} = @{$srclist{"FLUX"}}[@pntinds];
            @{$fhash{"errors"}} = @{$srclist{"FLUX_ERR"}}[@pntinds];
            @ltvarout = ltvar( \%fhash );
            $sumlist[$scnt]{"VAR_CHI2"}    = $ltvarout[0];
            $sumlist[$scnt]{"VAR_PROB"}    = $ltvarout[1];
            $sumlist[$scnt]{"FRATIO"}     = $ltvarout[2];
            $sumlist[$scnt]{"FRATIO_ERR"} = $ltvarout[3];
            $sumlist[$scnt]{"FLUXVAR"}    = $ltvarout[4];
        } else {
            $sumlist[$scnt]{"VAR_CHI2"}    = $mynull;
            $sumlist[$scnt]{"VAR_PROB"}    = $mynull;
            $sumlist[$scnt]{"FRATIO"}     = $mynull;
            $sumlist[$scnt]{"FRATIO_ERR"} = $mynull;
            $sumlist[$scnt]{"FLUXVAR"}    = $mynull;
        }

        # calculate EPIC band values
        for my $ie (1..$nbands) {                     # loop over energy bands
            # EPIC fluxes per energy band: weighted means of instrument values
            my $tmpsqerr;
            $tmpflux = 0.;
            $tmpfluxerr = 0.;
            for my $rowind (@srcinds[(grep
                              { ${$srclist{"ID_INST"}}[$srcinds[$_]] < 4
                             && ${$srclist{"ID_BAND"}}[$srcinds[$_]] eq $ie }
                                       0..$#srcinds)]) {
                if (defined(${srclist{"FLUX"}}[$rowind])
                    && ${srclist{"FLUX"}}[$rowind] < 1e99
                    && ${srclist{"FLUX_ERR"}}[$rowind] !=0 ) {
                    $tmpsqerr = 
                        1.0
                        /${srclist{"FLUX_ERR"}}[$rowind]
                        /${srclist{"FLUX_ERR"}}[$rowind];
                    $tmpflux 
                        += ${srclist{"FLUX"}}[$rowind]*$tmpsqerr;
                    $tmpfluxerr += $tmpsqerr;
                }
                # emldetect-like list: set flux to null and flux_err to zero,
                # if maskfrac below threshold
                if ((! defined ${$srclist{"MASKFRAC"}}[$rowind])
                    || (${$srclist{"MASKFRAC"}}[$rowind]<$maskthr)) {
                    ${$srclist{"FLUX"}}    [$rowind] = $mynull;
                    ${$srclist{"FLUX_ERR"}}[$rowind] = $mynull;
                }
            }
            if ($tmpfluxerr > 0) {
                $tmpflux /= $tmpfluxerr;
                $tmpfluxerr = 1.0/sqrt($tmpfluxerr);
            } else {
                $tmpflux = $mynull;
                $tmpfluxerr = 0.0;
            }
            # copy to source list
            $sumlist[$scnt]{"EP_".($ie)."_FLUX"}     = $tmpflux;
            $sumlist[$scnt]{"EP_".($ie)."_FLUX_ERR"} = $tmpfluxerr;

            ## VAR_CHI2*: measure for flux variability, if N_CONTRIB >= 2
            if ($sum_ncontrib > 1
                 && defined($tmpflux) && $tmpflux>0 && $tmpflux<1e99) {
                %fhash = ();
                $fhash{"meanflux"} = $tmpflux;
                ## all pointings
                for my $ii (0..$#tpinds) {
                    $fhash{"fluxes"}[$ii] = 
                        $sumlist[$tpinds[$ii]]{"EP_".($ie)."_FLUX"};
                    $fhash{"errors"}[$ii] = 
                        $sumlist[$tpinds[$ii]]{"EP_".($ie)."_FLUX_ERR"};
                }
                @ltvarout = ltvar( \%fhash );
                $sumlist[$scnt]{"VAR_CHI2_".($ie)}       = $ltvarout[0];
                $sumlist[$scnt]{"VAR_PROB_".($ie)}       = $ltvarout[1];
                $sumlist[$scnt]{"FRATIO_".($ie)}        = $ltvarout[2];
                $sumlist[$scnt]{"FRATIO_".($ie)."_ERR"} = $ltvarout[3];
                $sumlist[$scnt]{"FLUXVAR_".($ie)}       = $ltvarout[4];
            } else {
                $sumlist[$scnt]{"VAR_CHI2_".($ie)}       = $mynull;
                $sumlist[$scnt]{"VAR_PROB_".($ie)}       = $mynull;
                $sumlist[$scnt]{"FRATIO_".($ie)}        = $mynull;
                $sumlist[$scnt]{"FRATIO_".($ie)."_ERR"} = $mynull;
                $sumlist[$scnt]{"FLUXVAR_".($ie)}       = $mynull;
            }
        }                                           # end loop over energy bands
        # copy instrument band values
        for my $ii (1..$#insts) {
            for my $ib (1..$nbands) {
                my $eind =
                    @srcinds[(grep
                              { ${$srclist{"ID_INST"}}[$srcinds[$_]] eq $ii
                             && ${$srclist{"ID_BAND"}}[$srcinds[$_]] eq $ib    }
                              0..$#srcinds)];
                # instrument may be empty
                if (defined $eind) {
                    $sumlist[$scnt]{$insts[$ii]."_".$ib."_".$_} = 
                        ${$srclist{$suminfo{emlname}{$_}}}[$eind]
                        for ("FLUX"      ,
                             "FLUX_ERR"  ,
                             "RATE"      ,
                             "RATE_ERR"  ,
                             "DET_ML"    ,
                             "EXP"       ,
                             "BG"        ,
                             "VIG"       );
                } else {
		    # set to null explicitly, if instrument not defined
		    $sumlist[$scnt]{$insts[$ii]."_".$ib."_".$_} = 
			$mynull
			for ("FLUX"      ,
			     "FLUX_ERR"  ,
			     "RATE"      ,
			     "RATE_ERR"  ,
			     "CTS"       ,
			     "CTS_ERR"   ,
			     "DET_ML"    ,
			     "EXP"       ,
			     "BG"        ,
			     "VIG"       );
 		}
		# and if maskfrac below threshold
		if (! defined $sumlist[$scnt]{$insts[$ii]."_MASKFRAC"}
		    || $sumlist[$scnt]{$insts[$ii]."_MASKFRAC"}<$maskthr
		    || ! defined
		    $sumlist[$scnt]{$insts[$ii]."_".$ib."_EXP"} ||
		    $sumlist[$scnt]{$insts[$ii]."_".$ib."_EXP"}<=0
		    ) {
		    $sumlist[$scnt]{$insts[$ii]."_".$ib."_".$_} = $mynull
			for ("FLUX"     ,
			     "FLUX_ERR" ,
			     "RATE"     ,
			     "RATE_ERR" ,
			     "CTS"      ,
			     "CTS_ERR"  ,
			     "DET_ML"   ,
			     "VIG"      ); 
		}
		if (! defined
		    $sumlist[$scnt]{$insts[$ii]."_".$ib."_EXP"} ||
		    $sumlist[$scnt]{$insts[$ii]."_".$ib."_EXP"}<=0 
		    ) {
		    $sumlist[$scnt]{$insts[$ii]."_".$ib."_".$_} = 
			$mynull
			for ("EXP",
			     "BG" );
		}
            }
	    # and for the instrument values
	    if (! defined $sumlist[$scnt]{$insts[$ii]."_MASKFRAC"}
		|| $sumlist[$scnt]{$insts[$ii]."_MASKFRAC"}<$maskthr
		|| ! defined $sumlist[$scnt]{$insts[$ii]."_EXP"}
		|| $sumlist[$scnt]{$insts[$ii]."_EXP"}<=0
		) {
		$sumlist[$scnt]{$insts[$ii]."_".$_} = $mynull
		    for ("FLUX"      ,
			 "FLUX_ERR"  ,
			 "RATE"      ,
			 "RATE_ERR"  ,
			 "CTS"       ,
			 "CTS_ERR"   ,
			 "DET_ML"    );
	    }
	    if (! defined $sumlist[$scnt]{$insts[$ii]."_MASKFRAC"}
		|| $sumlist[$scnt]{$insts[$ii]."_MASKFRAC"} == 0
		|| ! defined $sumlist[$scnt]{$insts[$ii]."_EXP"}
		|| $sumlist[$scnt]{$insts[$ii]."_EXP"}<=0
		) {
		$sumlist[$scnt]{$insts[$ii]."_".$_} = 
		    $mynull
		    for ("EXP",
			 "BG" );
	    }

        }
        # MJD: minimum / maximum of pointing values
        # (assumption: either all are defined, or none)
        $sumlist[$scnt]{"MJD_FIRST"} = 
            (defined ${$srclist{$suminfo{emlname}{"MJD_FIRST"}}}[$pntinds[0]])
            ? List::Util::min
              @{$srclist{$suminfo{emlname}{"MJD_FIRST"}}}[@pntinds]
            : $mynull;
        $sumlist[$scnt]{"MJD_LAST"} = 
            (defined ${$srclist{$suminfo{emlname}{"MJD_LAST"}}}[$pntinds[0]])
            ? List::Util::max
              @{$srclist{$suminfo{emlname}{"MJD_LAST"}}}[@pntinds]
            : $mynull;
        # pointing information: none
        $sumlist[$scnt]{$_} = 
	    $mynulls{substr $suminfo{form}{$_},-1}
            for ("REVOLUT"   ,
                 "PA_PNT"    ,
		 "PN_SUBMODE",
                 "M1_SUBMODE",
                 "M2_SUBMODE",
                 "PN_FILTER" ,
                 "M1_FILTER" ,
                 "M2_FILTER" );
        # STACK_FLAG: integer representation of EP_FLAG in the all-stack row
        if ($sumlist[$scnt]{"EP_FLAG"} =~ /F|T/) {
            # 0, if all flags are "F"
            $sumlist[$scnt]{"STACK_FLAG"} = 0;
            # 1, if any of flags 1, 2, 3, 9 is / are true for a camera
            #    (~ close to a bad area)
            $sumlist[$scnt]{"STACK_FLAG"} = 1
                if (substr($sumlist[$scnt]{"EP_FLAG"}, 0, 3) =~ "T"
                    || substr($sumlist[$scnt]{"EP_FLAG"}, 8, 1) =~ "T");
            # 2, if flag 7 (any of flags 4-6) or 8 are true
            #    (~ probably spurious)
            $sumlist[$scnt]{"STACK_FLAG"} = 2
                if (substr($sumlist[$scnt]{"EP_FLAG"}, 6, 2) =~ "T");
            # 3, if all pointings have STACK_FLAG 2
            $sumlist[$scnt]{"STACK_FLAG"} = 3
                if (${$srclist{"STACK_FLAG"}}[$srcinds[0]] == $sum_ncontrib);   
        } else {
            $sumlist[$scnt]{"STACK_FLAG"} = $mynull;
        }

        ## And finally set flux values in the instrument summaries of the
        ## emldetect-like source lists to null, if mask fraction is below the
        ## limit, which was postponed to determine sums & variability
        @pntinds =
            @srcinds[(grep { 
                            substr(${$srclist{"ID_INST"}}[$srcinds[$_]],-1) eq 0
                                && ${$srclist{"ID_BAND"}}[$srcinds[$_]] eq 0 }
                      0..$#srcinds)];
        for my $rowind 
            (@srcinds[(grep { ${$srclist{"ID_INST"}}[$srcinds[$_]] > 4
                       && substr(${$srclist{"ID_BAND"}}[$srcinds[$_]],-1) eq 0 }
                          0..$#srcinds)])
        {
	    if (${$srclist{"MASKFRAC"}}[$rowind] < $maskthr
		|| ${$srclist{"EXP_MAP"}}[$rowind] <= 0.0 ) {
		${$srclist{"RATE"}}[$rowind] = $mynull;
		${$srclist{"RATE_ERR"}}[$rowind] = $mynull;
		${$srclist{"FLUX"}}[$rowind] = $mynull;
		${$srclist{"FLUX_ERR"}}[$rowind] = $mynull;
	    }

        }

        # done.

        }                      ### end if detection likelihood above lower limit
    }                                                  ### end loop over sources


    ##########################
    # write final source lists
    my $nvalid = $#srcvalinds+1;

    # emldetect-like format:
    # insert columns, removing quotes & whitespace, and write values
    for my $icol (1..$ncol) {

        if (defined $$listheader{"TUNIT".$icol}) {
            $emlfits
            ->insertcol({TTYPE => [($$listheader{"TTYPE".$icol} =~ s/\s|'//gr),
                                    $$listheader{COMMENTS}{"TTYPE".$icol}],
                         TFORM => [($$listheader{"TFORM".$icol} =~ s/\s|'//gr),
                                    $$listheader{COMMENTS}{"TFORM".$icol}],
                         TUNIT => [($$listheader{"TUNIT".$icol} =~ s/\s|'//gr),
                                    $$listheader{COMMENTS}{"TUNIT".$icol}]
                        })
            # add format and unit comments manually, since they seem to be
            # ignored within the insertcol command
            ->writekey("TUNIT".$icol, 
                       ($$listheader{"TUNIT".$icol} =~ s/\s|'//gr),
                        $$listheader{COMMENTS}{"TUNIT".$icol});
        } else {
            $emlfits
            ->insertcol({TTYPE => [($$listheader{"TTYPE".$icol} =~ s/\s|'//gr),
                                    $$listheader{COMMENTS}{"TTYPE".$icol}],
                         TFORM => [($$listheader{"TFORM".$icol} =~ s/\s|'//gr),
                                    $$listheader{COMMENTS}{"TFORM".$icol}]
                        });
        }
        @newvals = @{$srclist{$srcnames[$icol-1]}}[@srcvalinds];
        $emlfits
            ->writekey("TFORM".$icol, 
                       ($$listheader{"TFORM".$icol} =~ s/\s|'//gr),
                        $$listheader{COMMENTS}{"TFORM".$icol})
            ->writecol($srcnames[$icol-1], {rows=>[1,$nvalid]}, \@newvals);
        # null values in integer columns need to be set explicitly
        $emlfits->writekey("TNULL".$icol, 
                           $mynulls{$$listheader{"TTYPE".$icol}},
                           "Null value of the "
                           .$$listheader{"TTYPE".$icol} =~ s/\s|'//gr
                           ." column" )
            if ($$listheader{"TFORM".$icol} =~ s/\s|'//gr =~ /B|I|J/
                && grep { $_==$mynull} @{$srclist{$srcnames[$icol-1]}});
    }
    $emlfits->writekey("TDISP3", "F11.0", "Display ID_INST like integers"); 

    # catalogue-like format: row-wise
    # write columns
    for my $irow (0..$#sumlist) {
        # convert extent from image pixels to arcsec
        if (${$sumlist[$irow]}{"EXTENT"} > 0) {
            ${$sumlist[$irow]}{"EXTENT"    } *= $$fullheader{"XBINSIZE"};
            ${$sumlist[$irow]}{"EXTENT_ERR"} *= $$fullheader{"XBINSIZE"};
        }

        # write columns, setting undefined and infinite values to null
	# (converting strings to numerics for the test [only])
        for my $column (@sumnames) {
            if (defined ${$sumlist[$irow]}{$column}) {
		if ($suminfo{form}{$column} =~ /D|E/) {
		    $sumfits->writecol($column, 
				       {rows=>[$#sumlist-$irow+1], 
					nulval=>$mynull}, 
				       ${$sumlist[$irow]}{$column});
		} else {
		    $sumfits->writecol($column, 
				       {rows=>[$#sumlist-$irow+1]}, 
				       ${$sumlist[$irow]}{$column});
		}
	    } else {
		# nulls for different column types
		if ($suminfo{form}{$column} =~ /A/) {
		    $sumfits->writecol($column, 
				       {rows=>$#sumlist-$irow+1},
				       "");
		} else {   # implicit type casting ...
		    $sumfits->writecol($column, 
				       {rows=>$#sumlist-$irow+1},
				       $mynull);
		}
	    }
        }
    }
    # null values in integer columns need to be set explicitly
    for my $icol (0..$#sumnames) {
	$sumfits->writekey("TNULL".($icol+1), 
			   $mynulls{$suminfo{form}{$sumnames[$icol]}},
			   "Null value of column ".$sumnames[$icol])
	    if ($suminfo{form}{$sumnames[$icol]} =~ /B|I|J|K/);
    }


    # add number of detections and update history keyword of primary header
    ( @instinds ) = grep { ${$srclist{"ID_INST"}}[$_] ne 0
                               || ${srclist{"ID_BAND"}}[$_] ne 0 }
                    (@srcvalinds);
    ($sec,$min,$hour,$day,$mon,$year) = localtime();
    $emlfits
        ->move(-1)
        ->writekey("NDETECT",   $#srcvalinds-$#instinds,
                   "Total number of detections")
        ->writekey("LIKEMIN",   $mindetml,
                   "Minimum detection likelihood to include a source");
    $sumfits
        ->move(-1)
        ->writekey("NDETECT",   $#srcvalinds-$#instinds,
                   "Total number of detections")
        ->writekey("LIKEMIN",   $mindetml,
                   "Minimum detection likelihood to include a source");
    if (defined $hhead{"COMMENT"}) {
        $emlfits->writekey("COMMENT", $hhead{"COMMENT"}); 
        $sumfits->writekey("COMMENT", $hhead{"COMMENT"}); 
    }
    $emlfits->writekey("HISTORY","Updated by ".qx/edetect_stack -v/."at ".
                    sprintf("%04i-%02i-%02iT%02i:%02i:%02i",
                            $year+1900,$mon+1,$day,$hour,${min},${sec}));
    $sumfits->writekey("HISTORY","Updated by ".qx/edetect_stack -v/."at ".
                    sprintf("%04i-%02i-%02iT%02i:%02i:%02i",
                            $year+1900,$mon+1,$day,$hour,${min},${sec}));

    # close the files
    $emlfits->close();
    $sumfits->close();

    # information
    SAS::message($SAS::AppMsg, $SAS::VerboseMsg, 
             "Final source list comprises ".($#srcvalinds-$#instinds)
                 ." detections.");


################################################################################
### MERGE_FLAGS
# auxiliary subroutine to merge two dpssflag flags,
# true flags having preference over false ones (of course)

sub merge_flags {

    my @sone = split '', $_[0];
    my @stwo = split '', $_[1];
    my $mergedflag = join '', map { $sone[$_] gt $stwo[$_] 
                                        ? $sone[$_] 
                                        : $stwo[$_]         } 0..$#sone;
    return $mergedflag;

}                                                            ### END MERGE_FLAGS

################################################################################
### LTVAR
# Determine reduced chi2 and other measures of long-term variability during n
# contributing observations. Mean flux, individual fluxes and flux errors are
# passed as hash.
# output: VAR_CHI2 - VAR_PROB - FRATIO - FRATIO_ERR - FLUXVAR - minmax

sub ltvar {

    my %inhash = %{$_[0]};
    my $meanflux = $inhash{"meanflux"};
    my @fluxes   = @{$inhash{"fluxes"}};
    my @errors   = @{$inhash{"errors"}};

    # remove undefined values
    for my $ii (reverse(0..$#fluxes)) {
        if ((! defined $fluxes[$ii]) || (! defined $errors[$ii])
            || ($fluxes[$ii]<=0.0 || $fluxes[$ii]>1e99 || $errors[$ii]==0.0)) {
            splice @fluxes, $ii, 1;
            splice @errors, $ii, 1;
        }
    }

    # initialize result variables
    my $varchi = 0.0;
    my $vmmchi = 0.0;
    my $fratio = 0.0;
    my $fraerr = 0.0;
    my $fndiff = 0.0;
    my $varprob = 0**(-1);   ## $mynull

    ## chisquare
    if ($#fluxes > 0) {
        for my $ii (0..$#fluxes) {
            $varchi = $varchi
                + ( ($fluxes[$ii] - $meanflux) / $errors[$ii] )**2;
        }

	# varprob: formally same functions as likelihood:
	# 1 - chi2cdf(varchi*ndof,ndof) = exp(-like(ndof/2,varchi*ndof/2))

	$varprob = exp(-likelihood(.5*$#fluxes, .5*$varchi));

	# varchi
        $varchi /= $#fluxes;

        # minimum and maximum fluxes
        my $fmin = List::Util::min @fluxes;
        my $fmax = List::Util::max @fluxes;
        # flux errors taken at indices of min and max flux
        my @itmp;
        @itmp = grep { $fluxes[$_] ==  $fmin } (0..$#fluxes);
        my $emin = $errors[$itmp[0]];
        @itmp = grep { $fluxes[$_] ==  $fmax } (0..$#fluxes);
        my $emax = $errors[$itmp[0]];

        ## chisquare minimum - maximum
        $vmmchi = ( ($fmin - $meanflux) / $emin )**2
            + ( ($fmax - $meanflux) / $emax )**2;

        ## ratio of maximum to minimum flux and its error
        $fratio = ( $fmin > 0.0 ) ? $fmax / $fmin : 0.0;
        $fraerr = $fratio
            * sqrt( $emin*$emin/$fmin/$fmin + $emax*$emax/$fmax/$fmax );

        ## maximum error-normalized flux deviation 
        my @fnorm;
        my $nn = 0;
        for my $ii (0..$#fluxes) {
            for my $jj (($ii+1)..$#fluxes) {
                $fnorm[$nn] = 
                    abs($fluxes[$jj]-$fluxes[$ii]) /
                    sqrt( $errors[$ii]*$errors[$ii] +
                          $errors[$jj]*$errors[$jj] );
                $nn += 1;
            }
        }
        $fndiff = List::Util::max @fnorm;
    }

    return ($varchi, $varprob, $fratio, $fraerr, $fndiff, $vmmchi);

}                                                                 ### END LTVAR

################################################################################
### LIKELIHOOD
# return the negative natural logarithm of the complement of the incomplete
# gamma function: -ln(1-gammainc(arg1,arg2))
# WARNING: no sanity checks on input are performed
# Code follows emldetect/edetect_gen.f90/gammq

sub likelihood {

    # first argument: 0.5 * number of the degrees of freedom
    my $hdof = $_[0];
    # second argument: sum of individual likelihoods
    my $like = $_[1];
    # result: likelihood corresponding to two degrees of freedom
    my $ltwo;

    # no negative arguments
    if ($hdof <= 0. || $like <= 0.) {
        $ltwo = 0.;
    } else {

        # iteration / convergence criteria
        my $nmaxit = 1000;
        my $convrg = 3e-7;
        my $lowval = 1e-30;
        my $itres;

        # coefficients for approximation:
        my @coeff = (76.18009173,  -86.50532033,  24.01409822,
                     -1.231739516, 1.20858003e-3, -5.36382e-6);
        my $increm = 1.0;

        ## ln(gamma)
        $ltwo = ($hdof < 1.0) ? $hdof : $hdof-1.0;

        # integration 
        $increm += $coeff[$_]/($ltwo+$_+1.0) for (0..5);

        # intermediate value of $ltwo
        $ltwo = ($ltwo+0.5)*log($ltwo+5.5) - $ltwo-5.5 
            + log(2.50662827465*$increm);
        $ltwo -= log($hdof) if $hdof<1.0;

        ## iteration
        $like = $convrg if $like < $convrg;

        # branch 1:
        if ($like < $hdof+1.0) {
            $itres = 1/$hdof;
            my $del = $itres;
            my $niter = 0;
            do {
                $niter += 1;
                $del = $del * $like / ($hdof+$niter);
                $itres += $del;
            } until (abs($del)<$convrg*abs($itres) || $niter==$nmaxit);
        # result:
        $ltwo = -log( 1.0 - $itres * exp( $hdof*log($like) - $like - $ltwo ) );

        # branch 2:
        } else {
            $itres = 1.0;   # temporary start value, overwritten later-on
            my $itold = 0.0;
            my $a0 = 1.0;
            my $a1 = $like;
            my $b0 = 0.0;
            my $b1 = 1.0;
            my $fac = 1.0;

            my $niter = 0;
            do {
                $itold = $itres if ($a1 != 0);
                $niter += 1;
                $a0 = ($a1 + $a0*($niter-$hdof)) * $fac;
                $b0 = ($b1 + $b0*($niter-$hdof)) * $fac;
                $a1 = $like*$a0 + $niter*$fac*$a1;
                $b1 = $like*$b0 + $niter*$fac*$b1;
                if ($a1 != 0.0) {
                    $fac = 1.0/$a1;
                    $itres = $b1*$fac if ($b1 != 0 && $fac != 0);
                }
            } until (abs(($itres-$itold)/$itres)<$convrg || $niter==$nmaxit);
        # result:
        $ltwo = $like - $hdof*log($like) + $ltwo - log($itres);

        }

    }

    return $ltwo;

}                                                             ### END LIKELIHOOD

################################################################################

}

1;                                                      ### END STACK_SOURCELIST
